@@ -1,9 +1,9 @@
 // Copyright Contributors to the OpenVDB Project
-// SPDX-License-Identifier: MPL-2.0
+// SPDX-License-Identifier: Apache-2.0
 
 /// @file points/points.cc
 
-#include "openvdb.h"
+#include <openvdb/openvdb.h>
 #include "PointDataGrid.h"
 
 namespace openvdb {
