@@ -1,14 +1,16 @@
 // Copyright Contributors to the OpenVDB Project
-// SPDX-License-Identifier: MPL-2.0
+// SPDX-License-Identifier: Apache-2.0
 
 #include <openvdb/points/AttributeArray.h>
 #include <openvdb/points/AttributeSet.h>
 #include <openvdb/Types.h>
 #include <openvdb/math/Transform.h>
 #include <openvdb/io/File.h>
+#include <openvdb/util/Assert.h>
 
 #include <gtest/gtest.h>
 
+#ifdef OPENVDB_USE_DELAYED_LOADING
 #ifdef __clang__
 #pragma GCC diagnostic push
 #pragma GCC diagnostic ignored "-Wunused-macros"
@@ -20,13 +22,6 @@
 #endif
 #include <boost/interprocess/file_mapping.hpp>
 #include <boost/interprocess/mapped_region.hpp>
-#include <tbb/tick_count.h>
-
-#include <atomic>
-#include <cstdio> // for std::remove()
-#include <fstream>
-#include <sstream>
-#include <iostream>
 
 #ifdef _WIN32
 #include <boost/interprocess/detail/os_file_functions.hpp> // open_existing_file(), close_file()
@@ -38,8 +33,18 @@ namespace boost { namespace interprocess { namespace detail {} namespace ipcdeta
 #include <sys/types.h> // for struct stat
 #include <sys/stat.h> // for stat()
 #endif
+#endif // OPENVDB_USE_DELAYED_LOADING
+
+#include <tbb/tick_count.h>
+#include <atomic>
+
+#include <cstdio> // for std::remove()
+#include <fstream>
+#include <sstream>
+#include <iostream>
 
 
+#ifdef OPENVDB_USE_DELAYED_LOADING
 /// @brief io::MappedFile has a private constructor, so declare a class that acts as the friend
 class TestMappedFile
 {
@@ -49,6 +54,7 @@ public:
         return openvdb::SharedPtr<openvdb::io::MappedFile>(new openvdb::io::MappedFile(filename));
     }
 };
+#endif
 
 
 /// @brief Functionality similar to openvdb::util::CpuTimer except with prefix padding and no decimals.
@@ -887,8 +893,8 @@ struct VectorWrapper
     VectorWrapper(const T& _data) : data(_data) { }
     operator bool() const { return index < data.size(); }
     VectorWrapper& operator++() { index++; return *this; }
-    Index sourceIndex() const { assert(*this); return data[index].first; }
-    Index targetIndex() const { assert(*this); return data[index].second; }
+    Index sourceIndex() const { OPENVDB_ASSERT(*this); return data[index].first; }
+    Index targetIndex() const { OPENVDB_ASSERT(*this); return data[index].second; }
 
 private:
     const T& data;
@@ -1405,6 +1411,7 @@ TEST_F(TestAttributeArray, testStrided)
     }
 }
 
+#ifdef OPENVDB_USE_DELAYED_LOADING
 void
 TestAttributeArray::testDelayedLoad()
 {
@@ -2197,6 +2204,7 @@ TestAttributeArray::testDelayedLoad()
     }
 }
 TEST_F(TestAttributeArray, testDelayedLoad) { testDelayedLoad(); }
+#endif
 
 
 TEST_F(TestAttributeArray, testDefaultValue)
