@@ -1,5 +1,5 @@
 // Copyright Contributors to the OpenVDB Project
-// SPDX-License-Identifier: MPL-2.0
+// SPDX-License-Identifier: Apache-2.0
 
 #include <openvdb/Exceptions.h>
 #include <openvdb/openvdb.h>
@@ -9,7 +9,7 @@
 #include <openvdb/Grid.h>
 #include <openvdb/tree/Tree.h>
 #include <openvdb/util/CpuTimer.h>
-#include "gtest/gtest.h"
+#include <gtest/gtest.h>
 #include <iostream>
 #include <memory> // for std::make_unique
 
@@ -40,6 +40,11 @@ public:
     using Ptr = openvdb::SharedPtr<ProxyTree>;
     using ConstPtr = openvdb::SharedPtr<const ProxyTree>;
 
+    using Accessor = void;
+    using ConstAccessor = void;
+    using UnsafeAccessor = void;
+    using ConstUnsafeAccessor = void;
+
     static const openvdb::Index DEPTH;
     static const ValueType backg;
 
@@ -70,7 +75,11 @@ public:
     void prune(const ValueType& = 0) {}
     void clip(const openvdb::CoordBBox&) {}
     void clipUnallocatedNodes() override {}
+#if OPENVDB_ABI_VERSION_NUMBER >= 12
+    openvdb::Index64 unallocatedLeafCount() const override { return 0; }
+#else
     openvdb::Index32 unallocatedLeafCount() const override { return 0; }
+#endif
 
     void getIndexRange(openvdb::CoordBBox&) const override {}
     bool evalLeafBoundingBox(openvdb::CoordBBox& bbox) const override
@@ -83,10 +92,17 @@ public:
         { dim = openvdb::Coord(0, 0, 0); return false; }
 
     openvdb::Index treeDepth() const override { return 0; }
-    openvdb::Index leafCount() const override { return 0; }
+#if OPENVDB_ABI_VERSION_NUMBER >= 12
+    openvdb::Index64 leafCount() const override { return 0; }
+    std::vector<openvdb::Index64> nodeCount() const override
+        { return std::vector<openvdb::Index64>(DEPTH, 0); }
+    openvdb::Index64 nonLeafCount() const override { return 0; }
+#else
+    openvdb::Index32 leafCount() const override { return 0; }
     std::vector<openvdb::Index32> nodeCount() const override
         { return std::vector<openvdb::Index32>(DEPTH, 0); }
-    openvdb::Index nonLeafCount() const override { return 0; }
+    openvdb::Index32 nonLeafCount() const override { return 0; }
+#endif
     openvdb::Index64 activeVoxelCount() const override { return 0UL; }
     openvdb::Index64 inactiveVoxelCount() const override { return 0UL; }
     openvdb::Index64 activeLeafVoxelCount() const override { return 0UL; }
@@ -183,7 +199,6 @@ TEST_F(TestGrid, testIsTreeUnique)
     GridBase::Ptr grid3 = grid->copyGridWithNewTree();
     EXPECT_TRUE(grid->isTreeUnique());
 
-#if OPENVDB_ABI_VERSION_NUMBER >= 8
     // shallow copy using GridBase
     GridBase::Ptr grid4 = grid->copyGrid();
     EXPECT_TRUE(!grid4->isTreeUnique());
@@ -191,7 +206,6 @@ TEST_F(TestGrid, testIsTreeUnique)
     // copy with new tree using GridBase
     GridBase::Ptr grid5 = grid->copyGridWithNewTree();
     EXPECT_TRUE(grid5->isTreeUnique());
-#endif
 }
 
 
@@ -267,7 +281,7 @@ TEST_F(TestGrid, testCopyGrid)
     // shallow-copy a const grid but supply a new transform and meta map
     EXPECT_EQ(1.0, grid1->transform().voxelSize().x());
     EXPECT_EQ(size_t(0), grid1->metaCount());
-    EXPECT_EQ(Index(2), grid1->tree().leafCount());
+    EXPECT_EQ(Index64(2), grid1->tree().leafCount());
 
     math::Transform::Ptr xform(math::Transform::createLinearTransform(/*voxelSize=*/0.25));
     MetaMap meta;
@@ -280,7 +294,7 @@ TEST_F(TestGrid, testCopyGrid)
 
     EXPECT_EQ(0.25, grid3->transform().voxelSize().x());
     EXPECT_EQ(size_t(1), grid3->metaCount());
-    EXPECT_EQ(Index(2), tree3.leafCount());
+    EXPECT_EQ(Index64(2), tree3.leafCount());
     EXPECT_EQ(long(3), constGrid1->constTreePtr().use_count());
 }
 
@@ -514,3 +528,105 @@ TEST_F(TestGrid, testApply)
         EXPECT_EQ(4, n);
     }
 }
+
+TEST_F(TestGrid, testAdapter)
+{
+    openvdb::FloatGrid floatGrid;
+    const openvdb::FloatGrid constFloatGrid = floatGrid;
+    openvdb::FloatTree& floatTree = floatGrid.tree();
+    const openvdb::FloatTree& constFloatTree = floatGrid.constTree();
+    openvdb::tree::ValueAccessor<openvdb::FloatTree> floatAcc(floatGrid.tree());
+    openvdb::tree::ValueAccessor<const openvdb::FloatTree> constFloatAcc(floatGrid.constTree());
+
+    {
+        // test TreeAdapter<Tree>
+
+        using AdapterT = openvdb::TreeAdapter<openvdb::FloatTree>;
+
+        AdapterT::tree(floatTree);
+        AdapterT::tree(floatGrid);
+        AdapterT::tree(constFloatTree);
+        AdapterT::tree(constFloatGrid);
+        AdapterT::constTree(floatTree);
+        AdapterT::constTree(floatGrid);
+        AdapterT::constTree(constFloatTree);
+        AdapterT::constTree(constFloatGrid);
+
+        // test TreeAdapter<const Tree>
+
+        using ConstAdapterT = openvdb::TreeAdapter<const openvdb::FloatTree>;
+
+        ConstAdapterT::tree(floatTree);
+        ConstAdapterT::tree(floatGrid);
+        ConstAdapterT::tree(constFloatTree);
+        ConstAdapterT::tree(constFloatGrid);
+        ConstAdapterT::constTree(floatTree);
+        ConstAdapterT::constTree(floatGrid);
+        ConstAdapterT::constTree(constFloatTree);
+        ConstAdapterT::constTree(constFloatGrid);
+    }
+
+    {
+        // test TreeAdapter<Grid>
+
+        using AdapterT = openvdb::TreeAdapter<openvdb::FloatGrid>;
+
+        AdapterT::tree(floatTree);
+        AdapterT::tree(floatGrid);
+        AdapterT::tree(constFloatTree);
+        AdapterT::tree(constFloatGrid);
+        AdapterT::constTree(floatTree);
+        AdapterT::constTree(floatGrid);
+        AdapterT::constTree(constFloatTree);
+        AdapterT::constTree(constFloatGrid);
+
+        // test TreeAdapter<const Grid>
+
+        using ConstAdapterT = openvdb::TreeAdapter<const openvdb::FloatGrid>;
+
+        ConstAdapterT::tree(floatTree);
+        ConstAdapterT::tree(floatGrid);
+        ConstAdapterT::tree(constFloatTree);
+        ConstAdapterT::tree(constFloatGrid);
+        ConstAdapterT::constTree(floatTree);
+        ConstAdapterT::constTree(floatGrid);
+        ConstAdapterT::constTree(constFloatTree);
+        ConstAdapterT::constTree(constFloatGrid);
+    }
+
+    {
+        // test TreeAdapter<ValueAccessor<Tree>>
+
+        using AdapterT = openvdb::TreeAdapter<openvdb::tree::ValueAccessor<openvdb::FloatTree>>;
+
+        AdapterT::tree(floatTree);
+        AdapterT::tree(floatGrid);
+        AdapterT::tree(floatAcc);
+        AdapterT::tree(constFloatAcc);
+        AdapterT::tree(constFloatTree);
+        AdapterT::tree(constFloatGrid);
+        AdapterT::constTree(floatTree);
+        AdapterT::constTree(floatGrid);
+        AdapterT::constTree(floatAcc);
+        AdapterT::constTree(constFloatAcc);
+        AdapterT::constTree(constFloatTree);
+        AdapterT::constTree(constFloatGrid);
+
+        // test TreeAdapter<ValueAccessor<const Tree>>
+
+        using AdapterConstT = openvdb::TreeAdapter<openvdb::tree::ValueAccessor<const openvdb::FloatTree>>;
+
+        AdapterConstT::tree(floatTree);
+        AdapterConstT::tree(floatGrid);
+        AdapterConstT::tree(floatAcc);
+        AdapterConstT::tree(constFloatAcc);
+        AdapterConstT::tree(constFloatTree);
+        AdapterConstT::tree(constFloatGrid);
+        AdapterConstT::constTree(floatTree);
+        AdapterConstT::constTree(floatGrid);
+        AdapterConstT::constTree(floatAcc);
+        AdapterConstT::constTree(constFloatAcc);
+        AdapterConstT::constTree(constFloatTree);
+        AdapterConstT::constTree(constFloatGrid);
+    }
+}
