@@ -1,25 +1,23 @@
 // Copyright Contributors to the OpenVDB Project
-// SPDX-License-Identifier: MPL-2.0
-
-#include "gtest/gtest.h"
+// SPDX-License-Identifier: Apache-2.0
 
 #include <openvdb/points/PointDataGrid.h>
 #include <openvdb/openvdb.h>
+#include <openvdb/io/TempFile.h>
+#include <openvdb/math/Math.h>
 
 #include <openvdb/points/PointGroup.h>
 #include <openvdb/points/PointCount.h>
 #include <openvdb/points/PointConversion.h>
 
+#include <gtest/gtest.h>
+
 #include <cmath>
 #include <cstdio> // for std::remove()
 #include <cstdlib> // for std::getenv()
 #include <string>
 #include <vector>
 
-#ifdef _MSC_VER
-#include <windows.h>
-#endif
-
 using namespace openvdb;
 using namespace openvdb::points;
 
@@ -187,25 +185,8 @@ TEST_F(TestPointCount, testGroup)
     PointDataGrid::Ptr grid = createPointDataGrid<NullCodec, PointDataGrid>(positions, *transform);
     PointDataTree& tree = grid->tree();
 
-    // setup temp directory
-
-    std::string tempDir;
-    if (const char* dir = std::getenv("TMPDIR")) tempDir = dir;
-#ifdef _MSC_VER
-    if (tempDir.empty()) {
-        char tempDirBuffer[MAX_PATH+1];
-        int tempDirLen = GetTempPath(MAX_PATH+1, tempDirBuffer);
-        EXPECT_TRUE(tempDirLen > 0 && tempDirLen <= MAX_PATH);
-        tempDir = tempDirBuffer;
-    }
-#else
-    if (tempDir.empty()) tempDir = P_tmpdir;
-#endif
-
-    std::string filename;
-
     // check one leaf
-    EXPECT_EQ(tree.leafCount(), Index32(1));
+    EXPECT_EQ(tree.leafCount(), Index64(1));
 
     // retrieve first and last leaf attribute sets
 
@@ -303,14 +284,13 @@ TEST_F(TestPointCount, testGroup)
         EXPECT_EQ(pointCount(tree, ActiveFilter()), Index64(3));
         EXPECT_EQ(pointCount(tree, InactiveFilter()), Index64(1));
 
+        std::string filename;
+
         // write out grid to a temp file
         {
-            filename = tempDir + "/openvdb_test_point_load";
-
+            filename = "testPointCount1.vdb";
             io::File fileOut(filename);
-
             GridCPtrVec grids{grid};
-
             fileOut.write(grids);
         }
 
@@ -334,7 +314,9 @@ TEST_F(TestPointCount, testGroup)
 
             GroupFilter groupFilter("test", attributeSet);
 
-            bool inCoreOnly = true;
+            bool inCoreOnly;
+#ifdef OPENVDB_USE_DELAYED_LOADING
+            inCoreOnly = true;
 
             EXPECT_EQ(pointCount(inputTree, NullFilter(), inCoreOnly), Index64(0));
             EXPECT_EQ(pointCount(inputTree, ActiveFilter(), inCoreOnly), Index64(0));
@@ -344,6 +326,7 @@ TEST_F(TestPointCount, testGroup)
                 groupFilter, ActiveFilter()), inCoreOnly), Index64(0));
             EXPECT_EQ(pointCount(inputTree, BinaryFilter<GroupFilter, InactiveFilter>(
                 groupFilter, InactiveFilter()), inCoreOnly), Index64(0));
+#endif
 
             inCoreOnly = false;
 
@@ -357,6 +340,8 @@ TEST_F(TestPointCount, testGroup)
                 groupFilter, InactiveFilter()), inCoreOnly), Index64(1));
         }
 
+        std::remove(filename.c_str());
+
         // update the value mask and confirm point counts once again
 
         leafIter->updateValueMask();
@@ -386,7 +371,7 @@ TEST_F(TestPointCount, testGroup)
     grid = createPointDataGrid<NullCodec, PointDataGrid>(positions, *transform);
     PointDataTree& tree2 = grid->tree();
 
-    EXPECT_EQ(tree2.leafCount(), Index32(4));
+    EXPECT_EQ(tree2.leafCount(), Index64(4));
 
     leafIter = tree2.beginLeaf();
 
@@ -443,6 +428,15 @@ TEST_F(TestPointCount, testOffsets)
 {
     using namespace openvdb::math;
 
+    // empty tree
+    {
+        PointDataTree tree;
+        std::vector<Index64> offsets{ 10 };
+        Index64 total = pointOffsets(offsets, tree);
+        EXPECT_EQ(total, Index64(0));
+        EXPECT_TRUE(offsets.empty());
+    }
+
     const float voxelSize(1.0);
     math::Transform::Ptr transform(math::Transform::createLinearTransform(voxelSize));
 
@@ -525,34 +519,17 @@ TEST_F(TestPointCount, testOffsets)
         EXPECT_EQ(total, Index64(4));
     }
 
-    // setup temp directory
-
-    std::string tempDir;
-    if (const char* dir = std::getenv("TMPDIR")) tempDir = dir;
-#ifdef _MSC_VER
-    if (tempDir.empty()) {
-        char tempDirBuffer[MAX_PATH+1];
-        int tempDirLen = GetTempPath(MAX_PATH+1, tempDirBuffer);
-        EXPECT_TRUE(tempDirLen > 0 && tempDirLen <= MAX_PATH);
-        tempDir = tempDirBuffer;
-    }
-#else
-    if (tempDir.empty()) tempDir = P_tmpdir;
-#endif
-
     std::string filename;
 
     // write out grid to a temp file
     {
-        filename = tempDir + "/openvdb_test_point_load";
-
+        filename = "testPointCount1.vdb";
         io::File fileOut(filename);
-
         GridCPtrVec grids{grid};
-
         fileOut.write(grids);
     }
 
+#ifdef OPENVDB_USE_DELAYED_LOADING
     // test point offsets for a delay-loaded grid
     {
         io::File fileIn(filename);
@@ -595,6 +572,8 @@ TEST_F(TestPointCount, testOffsets)
         EXPECT_EQ(offsets[3], Index64(5));
         EXPECT_EQ(total, Index64(5));
     }
+#endif
+
     std::remove(filename.c_str());
 }
 
@@ -620,8 +599,8 @@ genPoints(std::vector<Vec3R>& positions, const int numPoints, const double scale
     // init
     math::Random01 randNumber(0);
     const int n = int(std::sqrt(double(numPoints)));
-    const double xScale = (2.0 * M_PI) / double(n);
-    const double yScale = M_PI / double(n);
+    const double xScale = (2.0 * openvdb::math::pi<double>()) / double(n);
+    const double yScale = openvdb::math::pi<double>() / double(n);
 
     double x, y, theta, phi;
     Vec3R pos;
