@@ -1,5 +1,5 @@
 // Copyright Contributors to the OpenVDB Project
-// SPDX-License-Identifier: MPL-2.0
+// SPDX-License-Identifier: Apache-2.0
 
 /// @file codegen/ComputeGenerator.cc
 
@@ -14,6 +14,8 @@
 #include "../compiler/CustomData.h"
 #include "../Exceptions.h"
 
+#include <openvdb/util/Assert.h>
+
 #include <llvm/ADT/SmallVector.h>
 #include <llvm/IR/CallingConv.h>
 #include <llvm/IR/Constants.h>
@@ -201,7 +203,7 @@ bool ComputeGenerator::visit(const ast::ConditionalStatement* cond)
         llvm::Value* condition = mValues.top(); mValues.pop();
 
         if (condition->getType()->isPointerTy()) {
-            condition = mBuilder.CreateLoad(condition);
+            condition = ir_load(mBuilder, condition);
         }
         llvm::Type* conditionType = condition->getType();
         // check the type of the condition branch is bool-convertable
@@ -248,7 +250,7 @@ bool ComputeGenerator::visit(const ast::TernaryOperator* tern)
     if (conditionSuccess) {
         // get the condition
         trueValue = mValues.top(); mValues.pop();
-        assert(trueValue);
+        OPENVDB_ASSERT(trueValue);
 
         trueType = trueValue->getType();
         truePtr = trueType->isPointerTy();
@@ -258,7 +260,7 @@ bool ComputeGenerator::visit(const ast::TernaryOperator* tern)
         // check the type of the condition branch is bool-convertable
         if (conditionType->isFloatingPointTy() || conditionType->isIntegerTy()) {
             boolCondition = truePtr ?
-                boolComparison(mBuilder.CreateLoad(trueValue), mBuilder) : boolComparison(trueValue, mBuilder);
+                boolComparison(ir_load(mBuilder, trueValue), mBuilder) : boolComparison(trueValue, mBuilder);
             mBuilder.CreateCondBr(boolCondition, trueBlock, falseBlock);
         }
         else {
@@ -295,7 +297,7 @@ bool ComputeGenerator::visit(const ast::TernaryOperator* tern)
 
     llvm::Value* falseValue = mValues.top(); mValues.pop();
     llvm::Type* falseType = falseValue->getType();
-    assert(trueType);
+    OPENVDB_ASSERT(trueType);
     // if both variables of same type do no casting or loading
     if (trueType != falseType) {
         // get the (contained) types of the expressions
@@ -308,14 +310,14 @@ bool ComputeGenerator::visit(const ast::TernaryOperator* tern)
         // if same contained type but one needs loading
         // can only have one pointer, one not, for scalars right now, i.e. no loaded arrays or strings
         if (trueType == falseType) {
-            assert(!(truePtr && falsePtr));
+            OPENVDB_ASSERT(!(truePtr && falsePtr));
             if (truePtr) {
                 mBuilder.SetInsertPoint(trueBranch);
-                trueValue = mBuilder.CreateLoad(trueValue);
+                trueValue = ir_load(mBuilder, trueValue);
             }
             else {
                 mBuilder.SetInsertPoint(falseBranch);
-                falseValue = mBuilder.CreateLoad(falseValue);
+                falseValue = ir_load(mBuilder, falseValue);
             }
         }
         else { // needs casting
@@ -326,15 +328,15 @@ bool ComputeGenerator::visit(const ast::TernaryOperator* tern)
             const bool trueScalar = (trueType->isIntegerTy() || trueType->isFloatingPointTy());
             if (trueScalar &&
                  (falseType->isIntegerTy() || falseType->isFloatingPointTy())) {
-                assert(trueType != falseType);
+                OPENVDB_ASSERT(trueType != falseType);
                 // SCALAR_SCALAR
                 returnType = typePrecedence(trueType, falseType);
                 // always load scalars here, even if they are the correct type
                 mBuilder.SetInsertPoint(trueBranch);
-                if (truePtr) trueValue = mBuilder.CreateLoad(trueValue);
+                if (truePtr) trueValue = ir_load(mBuilder, trueValue);
                 trueValue = arithmeticConversion(trueValue, returnType, mBuilder);
                 mBuilder.SetInsertPoint(falseBranch);
-                if (falsePtr) falseValue = mBuilder.CreateLoad(falseValue);
+                if (falsePtr) falseValue = ir_load(mBuilder, falseValue);
                 falseValue = arithmeticConversion(falseValue, returnType, mBuilder);
             }
             else if (trueType->isArrayTy() && falseType->isArrayTy()
@@ -357,7 +359,7 @@ bool ComputeGenerator::visit(const ast::TernaryOperator* tern)
                 // SCALAR_ARRAY
                 returnType = typePrecedence(trueType, falseType->getArrayElementType());
                 mBuilder.SetInsertPoint(trueBranch);
-                if (truePtr) trueValue = mBuilder.CreateLoad(trueValue);
+                if (truePtr) trueValue = ir_load(mBuilder, trueValue);
                 trueValue = arithmeticConversion(trueValue, returnType, mBuilder);
                 const size_t arraySize = falseType->getArrayNumElements();
                 if (arraySize == 9 || arraySize == 16) {
@@ -380,7 +382,7 @@ bool ComputeGenerator::visit(const ast::TernaryOperator* tern)
                     trueValue = arrayCast(trueValue, returnType, mBuilder);
                 }
                 mBuilder.SetInsertPoint(falseBranch);
-                if (falsePtr) falseValue = mBuilder.CreateLoad(falseValue);
+                if (falsePtr) falseValue = ir_load(mBuilder, falseValue);
                 falseValue = arithmeticConversion(falseValue, returnType, mBuilder);
                 const size_t arraySize = trueType->getArrayNumElements();
                 if (arraySize == 9 || arraySize == 16) {
@@ -429,7 +431,7 @@ bool ComputeGenerator::visit(const ast::Loop* loop)
     llvm::BasicBlock* postBodyBlock = conditionBlock;
 
     const ast::tokens::LoopToken loopType = loop->loopType();
-    assert((loopType == ast::tokens::LoopToken::FOR ||
+    OPENVDB_ASSERT((loopType == ast::tokens::LoopToken::FOR ||
             loopType == ast::tokens::LoopToken::WHILE ||
             loopType == ast::tokens::LoopToken::DO) &&
             "Unsupported loop type");
@@ -477,7 +479,7 @@ bool ComputeGenerator::visit(const ast::Loop* loop)
     if (this->traverse(loop->condition())) {
         llvm::Value* condition = mValues.top(); mValues.pop();
         if (condition->getType()->isPointerTy()) {
-            condition = mBuilder.CreateLoad(condition);
+            condition = ir_load(mBuilder, condition);
         }
         llvm::Type* conditionType = condition->getType();
         // check the type of the condition branch is bool-convertable
@@ -512,7 +514,7 @@ bool ComputeGenerator::visit(const ast::Loop* loop)
 bool ComputeGenerator::visit(const ast::Keyword* node)
 {
     const ast::tokens::KeywordToken keyw = node->keyword();
-    assert((keyw == ast::tokens::KeywordToken::RETURN ||
+    OPENVDB_ASSERT((keyw == ast::tokens::KeywordToken::RETURN ||
             keyw == ast::tokens::KeywordToken::BREAK  ||
             keyw == ast::tokens::KeywordToken::CONTINUE) &&
             "Unsupported keyword");
@@ -541,11 +543,11 @@ bool ComputeGenerator::visit(const ast::Keyword* node)
                 breakContinue = mBreakContinueStack.top();
 
             if (keyw == ast::tokens::KeywordToken::BREAK) {
-                assert(breakContinue.first);
+                OPENVDB_ASSERT(breakContinue.first);
                 mBuilder.CreateBr(breakContinue.first);
             }
             else if (keyw == ast::tokens::KeywordToken::CONTINUE) {
-                assert(breakContinue.second);
+                OPENVDB_ASSERT(breakContinue.second);
                 mBuilder.CreateBr(breakContinue.second);
             }
         }
@@ -573,7 +575,7 @@ bool ComputeGenerator::visit(const ast::BinaryOperator* node)
             lhs = mValues.top(); mValues.pop();
             llvm::Type* lhsType = lhs->getType();
             if (lhsType->isPointerTy()) {
-                lhs = mBuilder.CreateLoad(lhs);
+                lhs = ir_load(mBuilder, lhs);
                 lhsType = lhsType->getPointerElementType();
             }
 
@@ -601,7 +603,7 @@ bool ComputeGenerator::visit(const ast::BinaryOperator* node)
             llvm::Value* rhs = mValues.top(); mValues.pop();
             llvm::Type* rhsType = rhs->getType();
             if (rhsType->isPointerTy()) {
-                rhs = mBuilder.CreateLoad(rhs);
+                rhs = ir_load(mBuilder, rhs);
                 rhsType = rhsType->getPointerElementType();
             }
 
@@ -611,7 +613,7 @@ bool ComputeGenerator::visit(const ast::BinaryOperator* node)
 
                 mBuilder.SetInsertPoint(returnBlock);
                 if (lhsBranch) {// i.e. lhs was successful
-                    assert(rhs && lhs);
+                    OPENVDB_ASSERT(rhs && lhs);
                     llvm::PHINode* result = mBuilder.CreatePHI(LLVMType<bool>::get(mContext), 2, "binary_op");
                     result->addIncoming(lhs, lhsBranch->getParent());
                     result->addIncoming(rhs, rhsBranch->getParent());
@@ -667,7 +669,7 @@ bool ComputeGenerator::visit(const ast::UnaryOperator* node)
     if (type->isPointerTy()) {
         type = type->getPointerElementType();
         if (type->isIntegerTy() || type->isFloatingPointTy()) {
-            value = mBuilder.CreateLoad(value);
+            value = ir_load(mBuilder, value);
         }
     }
 
@@ -700,7 +702,7 @@ bool ComputeGenerator::visit(const ast::UnaryOperator* node)
         type = type->getArrayElementType();
         std::vector<llvm::Value*> elements;
         arrayUnpack(value, elements, mBuilder, /*load*/true);
-        assert(elements.size() > 0);
+        OPENVDB_ASSERT(elements.size() > 0);
 
         if (type->isIntegerTy()) {
             if (token == ast::tokens::MINUS) {
@@ -744,7 +746,7 @@ bool ComputeGenerator::visit(const ast::UnaryOperator* node)
         mLog.error("value is not a scalar or vector", node);
         return false;
     }
-    assert(result);
+    OPENVDB_ASSERT(result);
     mValues.pop();
     mValues.push(result);
     return true;
@@ -761,7 +763,7 @@ bool ComputeGenerator::visit(const ast::AssignExpression* assign)
     if (assign->isCompound()) {
         llvm::Value* rhsValue = nullptr;
         if (!this->binaryExpression(rhsValue, lhs, rhs, assign->operation(), assign)) return false;
-        assert(rhsValue);
+        OPENVDB_ASSERT(rhsValue);
         rhs = rhsValue;
         rhsType = rhs->getType();
     }
@@ -769,7 +771,7 @@ bool ComputeGenerator::visit(const ast::AssignExpression* assign)
     if (rhsType->isPointerTy()) {
         rhsType = rhsType->getPointerElementType();
         if (rhsType->isIntegerTy() || rhsType->isFloatingPointTy()) {
-            rhs = mBuilder.CreateLoad(rhs);
+            rhs = ir_load(mBuilder, rhs);
         }
     }
 
@@ -784,7 +786,7 @@ bool ComputeGenerator::visit(const ast::Crement* node)
         mLog.error("unable to assign to an rvalue", node);
         return false;
     }
-    llvm::Value* rvalue = mBuilder.CreateLoad(value);
+    llvm::Value* rvalue = ir_load(mBuilder, value);
     llvm::Type* type = rvalue->getType();
 
     if (type->isIntegerTy(1) || (!type->isIntegerTy() && !type->isFloatingPointTy())) {
@@ -794,7 +796,7 @@ bool ComputeGenerator::visit(const ast::Crement* node)
     }
     else {
         llvm::Value* crement = nullptr;
-        assert((node->increment() || node->decrement()) && "unrecognised crement operation");
+        OPENVDB_ASSERT((node->increment() || node->decrement()) && "unrecognised crement operation");
         if (node->increment())      crement = LLVMType<int32_t>::get(mContext, 1);
         else if (node->decrement()) crement = LLVMType<int32_t>::get(mContext, -1);
 
@@ -823,7 +825,7 @@ bool ComputeGenerator::visit(const ast::FunctionCall* node)
     }
     else {
         const size_t args = node->children();
-        assert(mValues.size() >= args);
+        OPENVDB_ASSERT(mValues.size() >= args);
 
         // initialize arguments. scalars are always passed by value, arrays
         // and strings always by pointer
@@ -838,12 +840,12 @@ bool ComputeGenerator::visit(const ast::FunctionCall* node)
                 type = type->getPointerElementType();
                 if (type->isIntegerTy() || type->isFloatingPointTy()) {
                     // pass by value
-                    arg = mBuilder.CreateLoad(arg);
+                    arg = ir_load(mBuilder, arg);
                 }
             }
             else {
                 // arrays should never be loaded
-                assert(!type->isArrayTy() && type != LLVMType<codegen::String>::get(mContext));
+                OPENVDB_ASSERT(!type->isArrayTy() && type != LLVMType<codegen::String>::get(mContext));
                 if (type->isIntegerTy() || type->isFloatingPointTy()) {
                     /*pass by value*/
                 }
@@ -858,7 +860,7 @@ bool ComputeGenerator::visit(const ast::FunctionCall* node)
         const Function* target = function->match(inputTypes, mContext, &match);
 
         if (!target) {
-            assert(!function->list().empty()
+            OPENVDB_ASSERT(!function->list().empty()
                    && "FunctionGroup has no function declarations");
 
             std::ostringstream os;
@@ -896,7 +898,7 @@ bool ComputeGenerator::visit(const ast::FunctionCall* node)
                 result = target->call(arguments, mBuilder, /*cast=*/false);
             }
 
-            assert(result && "Function has been invoked with no valid llvm Value return");
+            OPENVDB_ASSERT(result && "Function has been invoked with no valid llvm Value return");
             mValues.push(result);
         }
     }
@@ -923,13 +925,13 @@ bool ComputeGenerator::visit(const ast::Cast* node)
 
 
         if (value->getType()->isPointerTy()) {
-            value = mBuilder.CreateLoad(value);
+            value = ir_load(mBuilder, value);
         }
 
         if (targetType->isIntegerTy(1)) {
             // if target is bool, perform standard boolean conversion (*not* truncation).
             value = boolComparison(value, mBuilder);
-            assert(value->getType()->isIntegerTy(1));
+            OPENVDB_ASSERT(value->getType()->isIntegerTy(1));
         }
         else {
             value = arithmeticConversion(value, targetType, mBuilder);
@@ -965,7 +967,7 @@ bool ComputeGenerator::visit(const ast::DeclareLocal* node)
         value = insertStaticAlloca(mBuilder, type);
     }
 
-    assert(value);
+    OPENVDB_ASSERT(value);
     SymbolTable* current = mSymbolTables.getOrInsert(mScopeIndex);
 
     const std::string& name = node->local()->name();
@@ -996,7 +998,7 @@ bool ComputeGenerator::visit(const ast::DeclareLocal* node)
             if (initType->isPointerTy()) {
                 initType = initType->getPointerElementType();
                 if (initType->isIntegerTy() || initType->isFloatingPointTy()) {
-                    init = mBuilder.CreateLoad(init);
+                    init = ir_load(mBuilder, init);
                 }
             }
             if (!this->assignExpression(value, init, node)) return false;
@@ -1075,10 +1077,10 @@ bool ComputeGenerator::visit(const ast::ArrayUnpack* node)
     }
 
     if (component0->getType()->isPointerTy()) {
-        component0 = mBuilder.CreateLoad(component0);
+        component0 = ir_load(mBuilder, component0);
     }
     if (component1 && component1->getType()->isPointerTy()) {
-        component1 = mBuilder.CreateLoad(component1);
+        component1 = ir_load(mBuilder, component1);
     }
 
     if (!component0->getType()->isIntegerTy() ||
@@ -1100,18 +1102,18 @@ bool ComputeGenerator::visit(const ast::ArrayUnpack* node)
 
     llvm::Value* zero = LLVMType<int32_t>::get(mContext, 0);
     if (!component1) {
-        value = mBuilder.CreateGEP(value, {zero, component0});
+        value = ir_gep(mBuilder, value, {zero, component0});
     }
     else {
         // component0 = row, component1 = column. Index into the matrix array
         // which is layed out in row major = (component0*dim + component1)
-        assert(size == 9 || size == 16);
+        OPENVDB_ASSERT(size == 9 || size == 16);
         const int32_t dim = size == 9 ? 3 : 4;
         llvm::Value* offset =
             LLVMType<int32_t>::get(mContext, static_cast<int32_t>(dim));
         component0 = binaryOperator(component0, offset, ast::tokens::MULTIPLY, mBuilder);
         component0 = binaryOperator(component0, component1, ast::tokens::PLUS, mBuilder);
-        value = mBuilder.CreateGEP(value, {zero, component0});
+        value = ir_gep(mBuilder, value, {zero, component0});
     }
 
     mValues.push(value);
@@ -1133,7 +1135,7 @@ bool ComputeGenerator::visit(const ast::ArrayPack* node)
     for (size_t i = 0; i < num; ++i) {
         llvm::Value* value = mValues.top(); mValues.pop();
         if (value->getType()->isPointerTy()) {
-            value = mBuilder.CreateLoad(value);
+            value = ir_load(mBuilder, value);
         }
         if (value->getType()->isArrayTy()) {
             mLog.error("cannot build nested arrays", node->child(num-(i+1)));
@@ -1190,7 +1192,7 @@ bool ComputeGenerator::visit(const ast::Value<double>* node)
 
 bool ComputeGenerator::visit(const ast::Value<std::string>* node)
 {
-    assert(node->value().size() < static_cast<size_t>(std::numeric_limits<size_t>::max()));
+    OPENVDB_ASSERT(node->value().size() < static_cast<size_t>(std::numeric_limits<size_t>::max()));
     const FunctionGroup* axstring = this->getFunction("string::string", /*internal*/true);
     llvm::Value* loc = mBuilder.CreateGlobalStringPtr(node->value()); // char*
     llvm::Value* result = axstring->execute({loc}, mBuilder);
@@ -1203,7 +1205,7 @@ const FunctionGroup* ComputeGenerator::getFunction(const std::string &identifier
 {
     const FunctionGroup* F =
         mFunctionRegistry.getOrInsert(identifier, mOptions, allowInternal);
-    assert(F);
+    OPENVDB_ASSERT(F);
     return F;
 }
 
@@ -1229,7 +1231,7 @@ template <typename ValueType>
 typename std::enable_if<std::is_floating_point<ValueType>::value, bool>::type
 ComputeGenerator::visit(const ast::Value<ValueType>* node)
 {
-    assert(std::isinf(node->value()) || node->value() >= 0.0);
+    OPENVDB_ASSERT(std::isinf(node->value()) || node->value() >= 0.0);
     llvm::Constant* value = LLVMType<ValueType>::get(mContext, node->value());
     mValues.push(value);
     return true;
@@ -1247,10 +1249,10 @@ bool ComputeGenerator::visit(const ast::ExternalVariable* node)
     }
 
     llvm::Type* type = llvmTypeFromToken(node->type(), mContext);
-    llvm::Value* address = mBuilder.CreateLoad(ptrToAddress);
+    llvm::Value* address = ir_load(mBuilder, ptrToAddress);
     llvm::Value* value = mBuilder.CreateIntToPtr(address, type->getPointerTo(0));
     if (type->isIntegerTy() || type->isFloatingPointTy()) {
-        value = mBuilder.CreateLoad(value);
+        value = ir_load(mBuilder, value);
     }
     mValues.push(value);
     return true;
@@ -1267,7 +1269,7 @@ bool ComputeGenerator::visit(const ast::Tree*)
 
 bool ComputeGenerator::visit(const ast::Attribute*)
 {
-    assert(false && "Base ComputeGenerator attempted to generate code for an Attribute. "
+    OPENVDB_ASSERT(false && "Base ComputeGenerator attempted to generate code for an Attribute. "
         "PointComputeGenerator or VolumeComputeGenerator should be used for "
         "attribute accesses.");
     return false;
@@ -1298,7 +1300,7 @@ bool ComputeGenerator::assignExpression(llvm::Value* lhs, llvm::Value*& rhs, con
     if (lsize == 9 || lsize == 16) {
         if (rtype->isIntegerTy() || rtype->isFloatingPointTy()) {
             if (rhs->getType()->isPointerTy()) {
-                rhs = mBuilder.CreateLoad(rhs);
+                rhs = ir_load(mBuilder, rhs);
             }
             rhs = arithmeticConversion(rhs, ltype->getArrayElementType(), mBuilder);
             rhs = scalarToMatrix(rhs, mBuilder, lsize == 9 ? 3 : 4);
@@ -1314,7 +1316,7 @@ bool ComputeGenerator::assignExpression(llvm::Value* lhs, llvm::Value*& rhs, con
             return false;
         }
         else if (lsize == 1) {
-            assert(rsize > 1);
+            OPENVDB_ASSERT(rsize > 1);
             mLog.error("cannot assign a scalar value "
                 "from a vector or matrix. Consider using the [] operator to "
                 "get a particular element", node);
@@ -1332,7 +1334,7 @@ bool ComputeGenerator::assignExpression(llvm::Value* lhs, llvm::Value*& rhs, con
         (ltype->isFloatingPointTy() || ltype->isIntegerTy() || ltype->isArrayTy());
 
     if (componentwise) {
-        assert(rsize == lsize || (rsize == 1 || lsize == 1));
+        OPENVDB_ASSERT(rsize == lsize || (rsize == 1 || lsize == 1));
         const size_t resultsize = std::max(lsize, rsize);
 
         if (ltype != rtype) {
@@ -1357,12 +1359,12 @@ bool ComputeGenerator::assignExpression(llvm::Value* lhs, llvm::Value*& rhs, con
             if (!this->binaryExpression(newRhs, LLVMType<int32_t>::get(mContext, 0), rhs, ast::tokens::NOTEQUALS, node)) return false;
             if (!newRhs) return true;
             rhs = newRhs;
-            assert(newRhs->getType()->isIntegerTy(1));
+            OPENVDB_ASSERT(newRhs->getType()->isIntegerTy(1));
         }
 
         for (size_t i = 0; i < resultsize; ++i) {
-            llvm::Value* lelement = lsize == 1 ? lhs : mBuilder.CreateConstGEP2_64(lhs, 0, i);
-            llvm::Value* relement = rsize == 1 ? rhs : mBuilder.CreateLoad(mBuilder.CreateConstGEP2_64(rhs, 0, i));
+            llvm::Value* lelement = lsize == 1 ? lhs : ir_constgep2_64(mBuilder, lhs, 0, i);
+            llvm::Value* relement = rsize == 1 ? rhs : ir_load(mBuilder, ir_constgep2_64(mBuilder, rhs, 0, i));
             relement = arithmeticConversion(relement, opprec, mBuilder);
             mBuilder.CreateStore(relement, lelement);
         }
@@ -1417,7 +1419,7 @@ void ComputeGenerator::createFreeSymbolStrings(llvm::IRBuilder<>& B)
 
     for (llvm::BasicBlock& BB : *F) {
         llvm::Instruction* TI = BB.getTerminator();
-        assert(TI);
+        OPENVDB_ASSERT(TI);
         if (llvm::isa<llvm::ReturnInst>(TI)) {
             B.SetInsertPoint(TI);
             for (auto ptr : ptrs) {
@@ -1450,7 +1452,7 @@ bool ComputeGenerator::binaryExpression(llvm::Value*& result, llvm::Value* lhs,
     if (lsize == 9 || lsize == 16) {
         if (rtype->isIntegerTy() || rtype->isFloatingPointTy()) {
             if (rhs->getType()->isPointerTy()) {
-                rhs = mBuilder.CreateLoad(rhs);
+                rhs = ir_load(mBuilder, rhs);
             }
             rhs = arithmeticConversion(rhs, ltype->getArrayElementType(), mBuilder);
             rhs = scalarToMatrix(rhs, mBuilder, lsize == 9 ? 3 : 4);
@@ -1471,7 +1473,7 @@ bool ComputeGenerator::binaryExpression(llvm::Value*& result, llvm::Value* lhs,
     if (rsize == 9 || rsize == 16) {
         if (ltype->isIntegerTy() || ltype->isFloatingPointTy()) {
             if (lhs->getType()->isPointerTy()) {
-                lhs = mBuilder.CreateLoad(lhs);
+                lhs = ir_load(mBuilder, lhs);
             }
             lhs = arithmeticConversion(lhs, rtype->getArrayElementType(), mBuilder);
             lhs = scalarToMatrix(lhs, mBuilder, rsize == 9 ? 3 : 4);
@@ -1576,9 +1578,9 @@ bool ComputeGenerator::binaryExpression(llvm::Value*& result, llvm::Value* lhs,
 
     if (componentwise)
     {
-        assert(ltype->isArrayTy() || ltype->isFloatingPointTy() || ltype->isIntegerTy());
-        assert(rtype->isArrayTy() || rtype->isFloatingPointTy() || rtype->isIntegerTy());
-        assert(rsize == lsize || (rsize == 1 || lsize == 1));
+        OPENVDB_ASSERT(ltype->isArrayTy() || ltype->isFloatingPointTy() || ltype->isIntegerTy());
+        OPENVDB_ASSERT(rtype->isArrayTy() || rtype->isFloatingPointTy() || rtype->isIntegerTy());
+        OPENVDB_ASSERT(rsize == lsize || (rsize == 1 || lsize == 1));
 
         if (op == ast::tokens::DIVIDE || op == ast::tokens::MODULO) {
             if (llvm::Constant* c = llvm::dyn_cast<llvm::Constant>(rhs)) {
@@ -1619,12 +1621,12 @@ bool ComputeGenerator::binaryExpression(llvm::Value*& result, llvm::Value* lhs,
 
         if (!ltype->isArrayTy()) {
             if (lhs->getType()->isPointerTy()) {
-                lhs = mBuilder.CreateLoad(lhs);
+                lhs = ir_load(mBuilder, lhs);
             }
         }
         if (!rtype->isArrayTy()) {
             if (rhs->getType()->isPointerTy()) {
-                rhs = mBuilder.CreateLoad(rhs);
+                rhs = ir_load(mBuilder, rhs);
             }
         }
 
@@ -1641,15 +1643,15 @@ bool ComputeGenerator::binaryExpression(llvm::Value*& result, llvm::Value* lhs,
 
         if (op == ast::tokens::MODULO) {
             const FunctionGroup* mod = this->getFunction("floormod");
-            assert(mod);
+            OPENVDB_ASSERT(mod);
             target = mod->match({opprec,opprec}, mContext);
-            assert(target);
+            OPENVDB_ASSERT(target);
         }
 
         // perform op
         for (size_t i = 0; i < resultsize; ++i) {
-            llvm::Value* lelement = lsize == 1 ? lhs : mBuilder.CreateLoad(mBuilder.CreateConstGEP2_64(lhs, 0, i));
-            llvm::Value* relement = rsize == 1 ? rhs : mBuilder.CreateLoad(mBuilder.CreateConstGEP2_64(rhs, 0, i));
+            llvm::Value* lelement = lsize == 1 ? lhs : ir_load(mBuilder, ir_constgep2_64(mBuilder, lhs, 0, i));
+            llvm::Value* relement = rsize == 1 ? rhs : ir_load(mBuilder, ir_constgep2_64(mBuilder, rhs, 0, i));
             lelement = arithmeticConversion(lelement, opprec, mBuilder);
             relement = arithmeticConversion(relement, opprec, mBuilder);
             elements.emplace_back(runop(lelement, relement));
@@ -1661,7 +1663,7 @@ bool ComputeGenerator::binaryExpression(llvm::Value*& result, llvm::Value* lhs,
                 const ast::tokens::OperatorToken reductionOp =
                     op == ast::tokens::EQUALSEQUALS ? ast::tokens::AND : ast::tokens::OR;
                 result = elements.front();
-                assert(result->getType() == LLVMType<bool>::get(mContext));
+                OPENVDB_ASSERT(result->getType() == LLVMType<bool>::get(mContext));
                 for (size_t i = 1; i < resultsize; ++i) {
                     result = binaryOperator(result, elements[i], reductionOp, mBuilder);
                 }
@@ -1671,7 +1673,7 @@ bool ComputeGenerator::binaryExpression(llvm::Value*& result, llvm::Value* lhs,
                 result = insertStaticAlloca(mBuilder,
                     llvm::ArrayType::get(opprec, resultsize));
                 for (size_t i = 0; i < resultsize; ++i) {
-                    mBuilder.CreateStore(elements[i], mBuilder.CreateConstGEP2_64(result, 0, i));
+                    mBuilder.CreateStore(elements[i], ir_constgep2_64(mBuilder, result, 0, i));
                 }
             }
         }
