@@ -1,5 +1,5 @@
 // Copyright Contributors to the OpenVDB Project
-// SPDX-License-Identifier: MPL-2.0
+// SPDX-License-Identifier: Apache-2.0
 
 /// @file codegen/PointComputeGenerator.cc
 
@@ -14,6 +14,8 @@
 #include "openvdb_ax/Exceptions.h"
 #include "openvdb_ax/ast/Scanners.h"
 
+#include <openvdb/util/Assert.h>
+
 #include <llvm/ADT/SmallVector.h>
 #include <llvm/IR/BasicBlock.h>
 #include <llvm/IR/CallingConv.h>
@@ -144,15 +146,15 @@ inline void PointComputeGenerator::computePKBR(const AttributeRegistry&)
         [&](const std::vector<llvm::Value*>& args,
             llvm::IRBuilder<>& B) -> llvm::Value*
     {
-        assert(args.size() == 12);
+        OPENVDB_ASSERT(args.size() == 12);
         llvm::Value* vbuff = args[2]; //extractArgument(rangeFunction, "value_buffer");
         llvm::Value* abuff = args[3]; //extractArgument(rangeFunction, "active_buffer");
         llvm::Value* buffSize = args[4]; //extractArgument(rangeFunction, "buffer_size");
         llvm::Value* mode = args[5]; //extractArgument(rangeFunction, "mode");
-        assert(buffSize);
-        assert(vbuff);
-        assert(abuff);
-        assert(mode);
+        OPENVDB_ASSERT(buffSize);
+        OPENVDB_ASSERT(vbuff);
+        OPENVDB_ASSERT(abuff);
+        OPENVDB_ASSERT(mode);
 
         llvm::Function* base = B.GetInsertBlock()->getParent();
         llvm::LLVMContext& C = B.getContext();
@@ -169,7 +171,7 @@ inline void PointComputeGenerator::computePKBR(const AttributeRegistry&)
         // increment
         B.SetInsertPoint(iterBlock);
         {
-            llvm::Value* new_incr = B.CreateAdd(B.CreateLoad(incr), B.getInt64(1));
+            llvm::Value* new_incr = B.CreateAdd(ir_load(B, incr), B.getInt64(1));
             B.CreateStore(new_incr, incr);
             B.CreateBr(conditionBlock);
         }
@@ -177,7 +179,7 @@ inline void PointComputeGenerator::computePKBR(const AttributeRegistry&)
         // generate loop body
         B.SetInsertPoint(bodyBlock);
         {
-            llvm::Value* lincr = B.CreateLoad(incr);
+            llvm::Value* lincr = ir_load(B, incr);
 
             // Extract mask bit from array of words
             // NodeMask::isOn() = (0 != (mWords[n >> 6] & (Word(1) << (n & 63))));
@@ -185,8 +187,8 @@ inline void PointComputeGenerator::computePKBR(const AttributeRegistry&)
                 binaryOperator(lincr, B.getInt64(63), ast::tokens::BITAND, B),
                     ast::tokens::SHIFTLEFT, B);
             llvm::Value* word_idx = binaryOperator(lincr, B.getInt64(6), ast::tokens::SHIFTRIGHT, B);
-            llvm::Value* word = B.CreateGEP(abuff, word_idx);
-            word = B.CreateLoad(word);
+            llvm::Value* word = ir_gep(B, abuff, word_idx);
+            word = ir_load(B, word);
             word = binaryOperator(word, mask, ast::tokens::BITAND, B);
             llvm::Value* ison = B.CreateICmpNE(word, B.getInt64(0));
 
@@ -213,8 +215,8 @@ inline void PointComputeGenerator::computePKBR(const AttributeRegistry&)
 
                 // loops from pindex->pindexend (point grids have 32bit buffers)
                 llvm::Value* pindex = insertStaticAlloca(B, B.getInt32Ty());
-                llvm::Value* pindexend = B.CreateGEP(vbuff, lincr);
-                pindexend = B.CreateLoad(pindexend);
+                llvm::Value* pindexend = ir_gep(B, vbuff, lincr);
+                pindexend = ir_load(B, pindexend);
 
                 llvm::Value* firstvoxel = binaryOperator(lincr, B.getInt64(0), ast::tokens::EQUALSEQUALS, B);
                 B.CreateCondBr(firstvoxel, pthen, pelse);
@@ -227,20 +229,20 @@ inline void PointComputeGenerator::computePKBR(const AttributeRegistry&)
                 B.SetInsertPoint(pelse);
                 {
                     llvm::Value* prevv = binaryOperator(lincr, B.getInt64(1), ast::tokens::MINUS, B);
-                    llvm::Value* pindexcount = B.CreateGEP(vbuff, prevv);
-                    B.CreateStore(B.CreateLoad(pindexcount), pindex);
+                    llvm::Value* pindexcount = ir_gep(B, vbuff, prevv);
+                    B.CreateStore(ir_load(B, pindexcount), pindex);
                     B.CreateBr(pcondition);
                 }
 
                 B.SetInsertPoint(pcondition);
                 {
-                    llvm::Value* end = B.CreateICmpULT(B.CreateLoad(pindex), pindexend);
+                    llvm::Value* end = B.CreateICmpULT(ir_load(B, pindex), pindexend);
                     B.CreateCondBr(end, pbody, iterBlock);
                 }
 
                 B.SetInsertPoint(piter);
                 {
-                    llvm::Value* pnext = B.CreateAdd(B.CreateLoad(pindex), B.getInt32(1));
+                    llvm::Value* pnext = B.CreateAdd(ir_load(B, pindex), B.getInt32(1));
                     B.CreateStore(pnext, pindex);
                     B.CreateBr(pcondition);
                 }
@@ -253,7 +255,7 @@ inline void PointComputeGenerator::computePKBR(const AttributeRegistry&)
                         args[1],  // index space coordinate
                         vbuff,    // value buffer
                         ison,     // active/inactive
-                        arithmeticConversion(B.CreateLoad(pindex), B.getInt64Ty(), B),  // offset in the point array
+                        arithmeticConversion(ir_load(B, pindex), B.getInt64Ty(), B),  // offset in the point array
                         args[6],  // transforms
                         args[7],  // buffers
                         args[8],  // flags
@@ -269,7 +271,7 @@ inline void PointComputeGenerator::computePKBR(const AttributeRegistry&)
         }
 
         B.SetInsertPoint(conditionBlock);
-        llvm::Value* endCondition = B.CreateICmpULT(B.CreateLoad(incr), buffSize);
+        llvm::Value* endCondition = B.CreateICmpULT(ir_load(B, incr), buffSize);
 
         llvm::BasicBlock* postBlock = llvm::BasicBlock::Create(C, "k2.end", base);
         B.CreateCondBr(endCondition, bodyBlock, postBlock);
@@ -325,10 +327,10 @@ decode(llvm::Value* buffer,
     if (!codecs) {
         // Value can't be encoded so the buffer is guaranteed to be the decoded type
         buffer = B.CreatePointerCast(buffer, type->getPointerTo());
-        return B.CreateGEP(buffer, pid);
+        return ir_gep(B, buffer, pid);
     }
-    assert(!codecs->empty());
-    assert(store);
+    OPENVDB_ASSERT(!codecs->empty());
+    OPENVDB_ASSERT(store);
 
     llvm::Function* self = B.GetInsertBlock()->getParent();
     llvm::BasicBlock* post = llvm::BasicBlock::Create(C, "k1.get_buffer.decode", self);
@@ -351,13 +353,13 @@ decode(llvm::Value* buffer,
             // the input value and decode the value.
             const FunctionGroup* const F = codec->decoder();
             llvm::Type* encodedType = codec->decodedToEncoded(decodedType, C);
-            assert(encodedType);
+            OPENVDB_ASSERT(encodedType);
             encodedType = encodedType->getPointerTo();
 
             // guranteed to be castable
             llvm::Value* typedBuffer = B.CreatePointerCast(buffer, encodedType);
-            llvm::Value* encoded = B.CreateGEP(typedBuffer, pid);
-            assert(F->match({store->getType(), encoded->getType()}, C));
+            llvm::Value* encoded = ir_gep(B, typedBuffer, pid);
+            OPENVDB_ASSERT(F->match({store->getType(), encoded->getType()}, C));
             F->execute({store, encoded}, B);
             B.CreateBr(post);
         }
@@ -369,8 +371,8 @@ decode(llvm::Value* buffer,
     // @todo  We could instead register all vaid nullcodecs which would give
     //   guarantees should a codec not exist
     llvm::Value* typedBuffer = B.CreatePointerCast(buffer, type->getPointerTo());
-    llvm::Value* value = B.CreateGEP(typedBuffer, pid);
-    B.CreateStore(B.CreateLoad(value), store);
+    llvm::Value* value = ir_gep(B, typedBuffer, pid);
+    B.CreateStore(ir_load(B, value), store);
     B.CreateBr(post);
 
     B.SetInsertPoint(post);
@@ -400,10 +402,10 @@ encode(llvm::Value* in,
     if (!codecs) {
         // Value can't be encoded so the buffer is guaranteed to be the decoded type
         buffer = B.CreatePointerCast(buffer, type->getPointerTo());
-        B.CreateStore(B.CreateLoad(in), B.CreateGEP(buffer, pid));
+        B.CreateStore(ir_load(B, in), ir_gep(B, buffer, pid));
         return;
     }
-    assert(!codecs->empty());
+    OPENVDB_ASSERT(!codecs->empty());
 
     llvm::Function* self = B.GetInsertBlock()->getParent();
     llvm::BasicBlock* post = llvm::BasicBlock::Create(C, "k1.set_buffer.encode", self);
@@ -424,11 +426,11 @@ encode(llvm::Value* in,
         {
             const FunctionGroup* const F = codec->encoder();
             llvm::Type* encodedType = codec->decodedToEncoded(decodedType, C);
-            assert(encodedType);
+            OPENVDB_ASSERT(encodedType);
             encodedType = encodedType->getPointerTo();
             llvm::Value* typedBuffer = B.CreatePointerCast(buffer, encodedType);
-            llvm::Value* loc = B.CreateGEP(typedBuffer, pid);
-            assert(F->match({loc->getType(),in->getType()}, C));
+            llvm::Value* loc = ir_gep(B, typedBuffer, pid);
+            OPENVDB_ASSERT(F->match({loc->getType(),in->getType()}, C));
             F->execute({loc, in}, B);
             B.CreateBr(post);
         }
@@ -440,8 +442,8 @@ encode(llvm::Value* in,
     // @todo  We could instead register all vaid nullcodecs which would give
     //   guarantees should a codec not exist
     llvm::Value* typedBuffer = B.CreatePointerCast(buffer, type->getPointerTo());
-    llvm::Value* loc = B.CreateGEP(typedBuffer, pid);
-    B.CreateStore(B.CreateLoad(in), loc);
+    llvm::Value* loc = ir_gep(B, typedBuffer, pid);
+    B.CreateStore(ir_load(B, in), loc);
     B.CreateBr(post);
 
     B.SetInsertPoint(post);
@@ -455,15 +457,15 @@ inline void PointComputeGenerator::computePKB(const AttributeRegistry& registry)
         [&](const std::vector<llvm::Value*>& args,
             llvm::IRBuilder<>& B) -> llvm::Value*
     {
-        assert(args.size() == 11);
+        OPENVDB_ASSERT(args.size() == 11);
         auto& C = B.getContext();
         llvm::Function* self = B.GetInsertBlock()->getParent();
         llvm::Value* pindex = extractArgument(self, "point_index");
         llvm::Value* flags = extractArgument(self, "flags");
         llvm::Value* buffers = extractArgument(self, "buffers");
-        assert(buffers);
-        assert(pindex);
-        assert(flags);
+        OPENVDB_ASSERT(buffers);
+        OPENVDB_ASSERT(pindex);
+        OPENVDB_ASSERT(flags);
 
         // create array of void*. each pointer will encode an address to a stored typed value
         llvm::Type* locType = llvm::ArrayType::get(LLVMType<void*>::get(C), registry.data().size()); // [SIZE x i8*]
@@ -475,16 +477,16 @@ inline void PointComputeGenerator::computePKB(const AttributeRegistry& registry)
             const std::string token = data.tokenname();
             llvm::Type* type = llvmTypeFromToken(data.type(), C);
 
-            llvm::Value* decodedPtrs = B.CreateConstInBoundsGEP2_64(loc, 0, i++); // void**, location to hold the typed ptr
+            llvm::Value* decodedPtrs = ir_constinboundsgep2_64(B, loc, 0, i++); // void**, location to hold the typed ptr
             decodedPtrs = B.CreatePointerCast(decodedPtrs, type->getPointerTo()->getPointerTo()); // ValueType**
 
             llvm::Value* index = mModule.getGlobalVariable(token);
-            assert(index);
-            index = B.CreateLoad(index);
-            llvm::Value* buffer = B.CreateGEP(buffers, index);
-            buffer = B.CreateLoad(buffer); // void** = void*
+            OPENVDB_ASSERT(index);
+            index = ir_load(B, index);
+            llvm::Value* buffer = ir_gep(B, buffers, index);
+            buffer = ir_load(B, buffer); // void** = void*
 
-            llvm::Value* flag = B.CreateLoad(B.CreateGEP(flags, index));
+            llvm::Value* flag = ir_load(B, ir_gep(B, flags, index));
 
             // @todo  write handles shouldn't need to do this check
             llvm::Value* isuniform = B.CreateAnd(flag, LLVMType<uint64_t>::get(C, (uint64_t(1) << 63)));
@@ -527,7 +529,7 @@ inline void PointComputeGenerator::computePKB(const AttributeRegistry& registry)
         // invoke the point kernel for this value
         std::array<llvm::Value*, 11> input;
         std::copy_n(args.begin(), 11, input.begin());
-        input[6] = B.CreateConstInBoundsGEP2_64(loc, 0, 0); // void**, replace the buffers with the extracted values
+        input[6] = ir_constinboundsgep2_64(B, loc, 0, 0); // void**, replace the buffers with the extracted values
 
         B.CreateCall(compute, input);
 
@@ -543,17 +545,17 @@ inline void PointComputeGenerator::computePKB(const AttributeRegistry& registry)
             const std::string token = data.tokenname();
             llvm::Type* type = llvmTypeFromToken(data.type(), C);
 
-            llvm::Value* store = B.CreateConstInBoundsGEP2_64(loc, 0, i++); // void**, location to hold the typed ptr
-            store = B.CreateLoad(store); // void*
+            llvm::Value* store = ir_constinboundsgep2_64(B, loc, 0, i++); // void**, location to hold the typed ptr
+            store = ir_load(B, store); // void*
             store = B.CreatePointerCast(store, type->getPointerTo()); // ValueType*
 
             llvm::Value* index = mModule.getGlobalVariable(token);
-            assert(index);
-            index = B.CreateLoad(index);
-            llvm::Value* flag = B.CreateLoad(B.CreateGEP(flags, index));
+            OPENVDB_ASSERT(index);
+            index = ir_load(B, index);
+            llvm::Value* flag = ir_load(B, ir_gep(B, flags, index));
 
-            llvm::Value* buffer = B.CreateGEP(buffers, index);
-            buffer = B.CreateLoad(buffer); // void** = void*
+            llvm::Value* buffer = ir_gep(B, buffers, index);
+            buffer = ir_load(B, buffer); // void** = void*
 
             // The buffer should not be uniform if we're writing to it, so no
             // need to branch for this case (this should be guaranteed by the
@@ -613,13 +615,13 @@ inline void PointComputeGenerator::computePKAA(const AttributeRegistry& registry
         // insert the attribute into the map of global variables and get a unique global representing
         // the location which will hold the attribute handle offset.
         llvm::Value* index = M->getGlobalVariable(token);
-        assert(index);
-        index = B.CreateLoad(index);
+        OPENVDB_ASSERT(index);
+        index = ir_load(B, index);
 
         llvm::Value* arrays = extractArgument(self, "attribute_arrays");
-        assert(arrays);
-        llvm::Value* array = B.CreateGEP(arrays, index);
-        array = B.CreateLoad(array); // void** = void*
+        OPENVDB_ASSERT(arrays);
+        llvm::Value* array = ir_gep(B, arrays, index);
+        array = ir_load(B, array); // void** = void*
 
         // invoke C binding
         const bool usingString =
@@ -654,17 +656,17 @@ inline void PointComputeGenerator::computePKAA(const AttributeRegistry& registry
         // insert the attribute into the map of global variables and get a unique global representing
         // the location which will hold the attribute handle offset.
         llvm::Value* index = M->getGlobalVariable(token);
-        assert(index);
-        index = B.CreateLoad(index);
+        OPENVDB_ASSERT(index);
+        index = ir_load(B, index);
 
         llvm::Value* arrays = extractArgument(self, "attribute_arrays");
-        assert(arrays);
-        llvm::Value* array = B.CreateGEP(arrays, index);
-        array = B.CreateLoad(array); // void** = void*
+        OPENVDB_ASSERT(arrays);
+        llvm::Value* array = ir_gep(B, arrays, index);
+        array = ir_load(B, array); // void** = void*
 
         // load the result (if its a scalar)
         if (type->isIntegerTy() || type->isFloatingPointTy()) {
-            load = B.CreateLoad(load);
+            load = ir_load(B, load);
         }
         //llvm::errs() << "storing: " <<  *(load->getType()) << '\n';
 
@@ -680,7 +682,7 @@ inline void PointComputeGenerator::computePKAA(const AttributeRegistry& registry
 
         if (usingString) {
             llvm::Value* leafdata = extractArgument(self, "leaf_data");
-            assert(leafdata);
+            OPENVDB_ASSERT(leafdata);
             args.emplace_back(leafdata);
         }
 
@@ -694,7 +696,7 @@ inline void PointComputeGenerator::computePKAA(const AttributeRegistry& registry
         [&](const std::vector<llvm::Value*>& args,
             llvm::IRBuilder<>& B) -> llvm::Value*
     {
-        assert(args.size() == 11);
+        OPENVDB_ASSERT(args.size() == 11);
         auto& C = B.getContext();
         llvm::Function* self = B.GetInsertBlock()->getParent();
         llvm::Value* pindex = extractArgument(self, "point_index");
@@ -709,11 +711,11 @@ inline void PointComputeGenerator::computePKAA(const AttributeRegistry& registry
         size_t i = 0;
         for (const AttributeRegistry::AccessData& access : registry.data()) {
             llvm::Value* value = insertStaticAlloca(B, llvmTypeFromToken(access.type(), C));
-            assert(llvm::cast<llvm::AllocaInst>(value)->isStaticAlloca());
+            OPENVDB_ASSERT(llvm::cast<llvm::AllocaInst>(value)->isStaticAlloca());
             table.insert(access.tokenname(), value);
 
             // store the allocated ptr in the array of void*
-            llvm::Value* store = B.CreateConstInBoundsGEP2_64(loc, 0, i); // void**, location to hold the typed ptr
+            llvm::Value* store = ir_constinboundsgep2_64(B, loc, 0, i); // void**, location to hold the typed ptr
             value = B.CreatePointerCast(value, LLVMType<void*>::get(C));
             B.CreateStore(value, store);
 
@@ -730,7 +732,7 @@ inline void PointComputeGenerator::computePKAA(const AttributeRegistry& registry
         // invoke the point kernel for this value
         std::array<llvm::Value*, 11> input;
         std::copy_n(args.begin(), 11, input.begin());
-        input[6] = B.CreateConstInBoundsGEP2_64(loc, 0, 0); // void**, replace the buffers with the extracted values
+        input[6] = ir_constinboundsgep2_64(B, loc, 0, 0); // void**, replace the buffers with the extracted values
         B.CreateCall(compute, input);
 
         // insert set code and deallocations
@@ -740,7 +742,7 @@ inline void PointComputeGenerator::computePKAA(const AttributeRegistry& registry
             const std::string token = data.tokenname();
             llvm::Value* value = table.get(token);
             // // Expected to be used more than one (i.e. should never be zero)
-            // assert(value->hasNUsesOrMore(1));
+            // OPENVDB_ASSERT(value->hasNUsesOrMore(1));
             // // Check to see if this value is still being used - it may have
             // // been cleaned up due to returns. If there's only one use, it's
             // // the original get of this attribute.
@@ -862,12 +864,12 @@ bool PointComputeGenerator::visit(const ast::Attribute* node)
     llvm::Value* index = mModule.getGlobalVariable(node->tokenname());
     llvm::Type* type = llvmTypeFromToken(node->type(), mContext);
 
-    assert(index);
+    OPENVDB_ASSERT(index);
     // index into the void* array of handles and load the value.
-    index = mBuilder.CreateLoad(index);
+    index = ir_load(mBuilder, index);
     llvm::Value* value = extractArgument(mFunction, "values"); // void**
-    value = mBuilder.CreateGEP(value, index); // void**
-    value = mBuilder.CreateLoad(value); // void*
+    value = ir_gep(mBuilder, value, index); // void**
+    value = ir_load(mBuilder, value); // void*
     value = mBuilder.CreatePointerCast(value, type->getPointerTo()); // void* = ValueType*
 
     mValues.push(value);
