@@ -1,5 +1,5 @@
 // Copyright Contributors to the OpenVDB Project
-// SPDX-License-Identifier: MPL-2.0
+// SPDX-License-Identifier: Apache-2.0
 
 /// @file codegen/VolumeComputeGenerator.cc
 
@@ -9,6 +9,8 @@
 #include "Types.h"
 #include "Utils.h"
 
+#include <openvdb/util/Assert.h>
+
 #include "../Exceptions.h"
 #include "../ast/Scanners.h"
 
@@ -91,15 +93,15 @@ inline void VolumeComputeGenerator::computek2(llvm::Function* compute, const Att
         [&](const std::vector<llvm::Value*>& args,
             llvm::IRBuilder<>& B) -> llvm::Value*
     {
-        assert(args.size() == 9);
+        OPENVDB_ASSERT(args.size() == 9);
         llvm::Value* vbuff = args[2]; //extractArgument(rangeFunction, "value_buffer");
         llvm::Value* abuff = args[3]; //extractArgument(rangeFunction, "active_buffer");
         llvm::Value* buffSize = args[4]; //extractArgument(rangeFunction, "buffer_size");
         llvm::Value* mode = args[5]; //extractArgument(rangeFunction, "mode");
-        assert(buffSize);
-        assert(vbuff);
-        assert(abuff);
-        assert(mode);
+        OPENVDB_ASSERT(buffSize);
+        OPENVDB_ASSERT(vbuff);
+        OPENVDB_ASSERT(abuff);
+        OPENVDB_ASSERT(mode);
 
         llvm::Function* base = B.GetInsertBlock()->getParent();
         llvm::LLVMContext& C = B.getContext();
@@ -115,13 +117,13 @@ inline void VolumeComputeGenerator::computek2(llvm::Function* compute, const Att
 
         // increment
         B.SetInsertPoint(iterBlock);
-        llvm::Value* new_incr = B.CreateAdd(B.CreateLoad(incr), B.getInt64(1));
+        llvm::Value* new_incr = B.CreateAdd(ir_load(B, incr), B.getInt64(1));
         B.CreateStore(new_incr, incr);
         B.CreateBr(conditionBlock);
 
         // generate loop body
         B.SetInsertPoint(bodyBlock);
-        llvm::Value* lincr = B.CreateLoad(incr);
+        llvm::Value* lincr = ir_load(B, incr);
 
         // Extract mask bit from array of words
         // NodeMask::isOn() = (0 != (mWords[n >> 6] & (Word(1) << (n & 63))));
@@ -129,8 +131,8 @@ inline void VolumeComputeGenerator::computek2(llvm::Function* compute, const Att
             binaryOperator(lincr, B.getInt64(63), ast::tokens::BITAND, B),
                 ast::tokens::SHIFTLEFT, B);
         llvm::Value* word_idx = binaryOperator(lincr, B.getInt64(6), ast::tokens::SHIFTRIGHT, B);
-        llvm::Value* word = B.CreateGEP(abuff, word_idx);
-        word = B.CreateLoad(word);
+        llvm::Value* word = ir_gep(B, abuff, word_idx);
+        word = ir_load(B, word);
         word = binaryOperator(word, mask, ast::tokens::BITAND, B);
         llvm::Value* ison = B.CreateICmpNE(word, B.getInt64(0));
 
@@ -152,7 +154,7 @@ inline void VolumeComputeGenerator::computek2(llvm::Function* compute, const Att
                 args[1],            // index space coordinate
                 vbuff,              // value buffer
                 ison,               // active/inactive
-                B.CreateLoad(incr), // offset in the value buffer
+                ir_load(B, incr), // offset in the value buffer
                 args[6],            // read accessors
                 args[7],            // transforms
                 args[8]             // write index
@@ -162,7 +164,7 @@ inline void VolumeComputeGenerator::computek2(llvm::Function* compute, const Att
         }
 
         B.SetInsertPoint(conditionBlock);
-        llvm::Value* endCondition = B.CreateICmpULT(B.CreateLoad(incr), buffSize);
+        llvm::Value* endCondition = B.CreateICmpULT(ir_load(B, incr), buffSize);
 
         llvm::BasicBlock* postBlock = llvm::BasicBlock::Create(C, "k2.end", base);
         B.CreateCondBr(endCondition, bodyBlock, postBlock);
@@ -203,7 +205,7 @@ inline void VolumeComputeGenerator::computek3(llvm::Function* compute, const Att
         [&, this](const std::vector<llvm::Value*>& args,
             llvm::IRBuilder<>& B) -> llvm::Value*
     {
-        assert(args.size() == 6);
+        OPENVDB_ASSERT(args.size() == 6);
         llvm::Value* isc = args[1]; // index space coord
         llvm::Value* wi = args[4]; // write index
         llvm::Value* wa = args[5]; // write_accessor
@@ -219,8 +221,8 @@ inline void VolumeComputeGenerator::computek3(llvm::Function* compute, const Att
             type = type->getPointerElementType();
 
             llvm::Value* registeredIndex = this->mModule.getGlobalVariable(token);
-            assert(registeredIndex);
-            registeredIndex = B.CreateLoad(registeredIndex);
+            OPENVDB_ASSERT(registeredIndex);
+            registeredIndex = ir_load(B, registeredIndex);
             llvm::Value* result = B.CreateICmpEQ(wi, registeredIndex);
 
             llvm::BasicBlock* thenBlock = llvm::BasicBlock::Create(C, "k3.invoke_k1_" + token, base);
@@ -234,7 +236,7 @@ inline void VolumeComputeGenerator::computek3(llvm::Function* compute, const Att
 
             const FunctionGroup* const F = this->getFunction("probevalue", true);
             F->execute({wa, isc, ison, location}, B);
-            ison = B.CreateLoad(ison);
+            ison = ir_load(B, ison);
 
             llvm::Value* vptr = B.CreatePointerCast(location, LLVMType<void*>::get(C));
 
@@ -252,7 +254,7 @@ inline void VolumeComputeGenerator::computek3(llvm::Function* compute, const Att
 
             // set the voxel - load the result (if its a scalar)
             if (type->isIntegerTy() || type->isFloatingPointTy()) {
-                location = B.CreateLoad(location);
+                location = ir_load(B, location);
             }
 
             const FunctionGroup* const function = this->getFunction("setvoxel", true);
@@ -337,13 +339,13 @@ AttributeRegistry::Ptr VolumeComputeGenerator::generate(const ast::Tree& tree)
         {
             llvm::Value* vptr = mBuilder.CreateAlloca(type->getPointerTo(0));
             localTable->insert(data.tokenname() + "_vptr", vptr);
-            assert(llvm::cast<llvm::AllocaInst>(vptr)->isStaticAlloca());
+            OPENVDB_ASSERT(llvm::cast<llvm::AllocaInst>(vptr)->isStaticAlloca());
         }
 
         // @warning This method will insert the alloc before the above alloc.
         //  This is fine, but is worth noting
         llvm::Value* value = insertStaticAlloca(mBuilder, type);
-        assert(llvm::cast<llvm::AllocaInst>(value)->isStaticAlloca());
+        OPENVDB_ASSERT(llvm::cast<llvm::AllocaInst>(value)->isStaticAlloca());
 
         // @note  this technically doesn't need to live in the local table
         //  (only the pointer to this value (_vptr) needs to) but it's
@@ -381,8 +383,8 @@ bool VolumeComputeGenerator::visit(const ast::Attribute* node)
     const std::string globalName = node->tokenname();
     llvm::Value* value;
     value = localTable->get(globalName + "_vptr");
-    value = mBuilder.CreateLoad(value);
-    assert(value);
+    value = ir_load(mBuilder, value);
+    OPENVDB_ASSERT(value);
     mValues.push(value);
     return true;
 }
@@ -400,7 +402,7 @@ void VolumeComputeGenerator::getAccessorValue(const std::string& globalName, llv
     llvm::Value* registeredIndex = llvm::cast<llvm::GlobalVariable>
         (mModule.getOrInsertGlobal(globalName, LLVMType<int64_t>::get(mContext)));
     this->globals().insert(globalName, registeredIndex);
-    registeredIndex = mBuilder.CreateLoad(registeredIndex);
+    registeredIndex = ir_load(mBuilder, registeredIndex);
 
     // first see if pre cached node exists.
 
@@ -417,12 +419,12 @@ void VolumeComputeGenerator::getAccessorValue(const std::string& globalName, llv
     {
         llvm::Value* valueptr = extractArgument(mFunction, "value");
         llvm::Value* offset = extractArgument(mFunction, "offset");
-        assert(valueptr);
-        assert(offset);
+        OPENVDB_ASSERT(valueptr);
+        OPENVDB_ASSERT(offset);
 
         llvm::Type* type = location->getType(); // ValueType*
         valueptr = mBuilder.CreatePointerCast(valueptr, type);
-        llvm::Value* value = mBuilder.CreateGEP(valueptr, offset);
+        llvm::Value* value = ir_gep(mBuilder, valueptr, offset);
         mBuilder.CreateStore(value, this->mSymbolTables.get(1)->get(globalName + "_vptr"));
         mBuilder.CreateBr(post);
     }
@@ -436,18 +438,18 @@ void VolumeComputeGenerator::getAccessorValue(const std::string& globalName, llv
         llvm::Value* transformPtr = extractArgument(mFunction, "transforms");
         llvm::Value* origin = extractArgument(mFunction, "origin");
         llvm::Value* offset = extractArgument(mFunction, "offset");
-        assert(accessorPtr);
-        assert(transformPtr);
-        assert(origin);
-        assert(offset);
+        OPENVDB_ASSERT(accessorPtr);
+        OPENVDB_ASSERT(transformPtr);
+        OPENVDB_ASSERT(origin);
+        OPENVDB_ASSERT(offset);
 
-        accessorPtr = mBuilder.CreateGEP(accessorPtr, registeredIndex);
-        llvm::Value* targetTransform = mBuilder.CreateGEP(transformPtr, registeredIndex);
-        llvm::Value* sourceTransform = mBuilder.CreateGEP(transformPtr, accessIndex);
+        accessorPtr = ir_gep(mBuilder, accessorPtr, registeredIndex);
+        llvm::Value* targetTransform = ir_gep(mBuilder, transformPtr, registeredIndex);
+        llvm::Value* sourceTransform = ir_gep(mBuilder, transformPtr, accessIndex);
 
-        llvm::Value* accessor = mBuilder.CreateLoad(accessorPtr);
-        targetTransform = mBuilder.CreateLoad(targetTransform);
-        sourceTransform = mBuilder.CreateLoad(sourceTransform);
+        llvm::Value* accessor = ir_load(mBuilder, accessorPtr);
+        targetTransform = ir_load(mBuilder, targetTransform);
+        sourceTransform = ir_load(mBuilder, sourceTransform);
 
         const FunctionGroup* const F = this->getFunction("getvoxel", true);
         F->execute({accessor, sourceTransform, targetTransform, origin, offset, location}, mBuilder);
@@ -467,17 +469,17 @@ llvm::Value* VolumeComputeGenerator::accessorHandleFromToken(const std::string&
         (mModule.getOrInsertGlobal(globalName, LLVMType<int64_t>::get(mContext)));
     this->globals().insert(globalName, registeredIndex);
 
-    registeredIndex = mBuilder.CreateLoad(registeredIndex);
+    registeredIndex = ir_load(mBuilder, registeredIndex);
 
     // index into the void* array of handles and load the value.
     // The result is a loaded void* value
 
     llvm::Value* accessorPtr = extractArgument(mFunction, "accessors");
-    assert(accessorPtr);
-    accessorPtr = mBuilder.CreateGEP(accessorPtr, registeredIndex);
+    OPENVDB_ASSERT(accessorPtr);
+    accessorPtr = ir_gep(mBuilder, accessorPtr, registeredIndex);
 
     // return loaded void** = void*
-    return mBuilder.CreateLoad(accessorPtr);
+    return ir_load(mBuilder, accessorPtr);
 }
 
 ///////////////////////////////////////////////////////////////////////////
