@@ -1,13 +1,15 @@
 // Copyright Contributors to the OpenVDB Project
-// SPDX-License-Identifier: MPL-2.0
+// SPDX-License-Identifier: Apache-2.0
 
 /// @file compiler/PointExecutable.cc
 
+#include "cli.h" // from vdb_ax command line tool
+
 #include "PointExecutable.h"
 #include "Logger.h"
 
-#include "openvdb_ax/ast/Scanners.h"
 #include "openvdb_ax/Exceptions.h"
+#include "openvdb_ax/ast/Scanners.h"
 // @TODO refactor so we don't have to include PointComputeGenerator.h,
 // but still have the functions defined in one place
 #include "openvdb_ax/codegen/PointComputeGenerator.h"
@@ -15,6 +17,7 @@
 #include "openvdb_ax/codegen/Codecs.h"
 
 #include <openvdb/Types.h>
+#include <openvdb/util/Assert.h>
 
 #include <openvdb/points/AttributeArray.h>
 #include <openvdb/points/PointAttribute.h>
@@ -31,14 +34,120 @@ namespace OPENVDB_VERSION_NAME {
 
 namespace ax {
 
+/// @brief Settings which are stored on the point executer
+///   and are configurable by the user.
+template <bool IsCLI>
 struct PointExecutable::Settings
 {
-    bool mCreateMissing = true;
-    size_t mGrainSize = 1;
-    std::string mGroup = "";
     //IterType mValueIterator = IterType::ON;
-    bool mPostDelete = false;
-    AttributeBindings mBindings;
+
+    template <typename T>
+    using ParamT = typename std::conditional<IsCLI,
+        cli::Param<T>,
+        cli::BasicParam<T>>::type;
+
+    template <typename T>
+    using ParamBuilderT = typename std::conditional<IsCLI,
+        cli::ParamBuilder<T>,
+        cli::BasicParamBuilder<T>>::type;
+
+
+    ///////////////////////////////////////////////////////////////////////////
+
+    inline std::vector<cli::ParamBase*> optional()
+    {
+        OPENVDB_ASSERT(IsCLI);
+        std::vector<cli::ParamBase*> params {
+            &this->mCreateMissing,
+            &this->mGroup,
+            &this->mGrainSize,
+            &this->mBindings
+        };
+        return params;
+    }
+
+    inline void init(const PointExecutable::Settings<true>& S)
+    {
+        if (S.mCreateMissing.isInit())  mCreateMissing = S.mCreateMissing.get();
+        if (S.mGroup.isInit())          mGroup = S.mGroup.get();
+        if (S.mGrainSize.isInit())      mGrainSize = S.mGrainSize.get();
+        if (S.mBindings.isInit())       mBindings = S.mBindings.get();
+    }
+
+    ///////////////////////////////////////////////////////////////////////////
+
+    ParamT<bool> mCreateMissing =
+        ParamBuilderT<bool>()
+            .addOpt("--create-missing [ON|OFF]")
+            .setDoc("whether to implicitly point attributes (Default: ON). Attributes are "
+                    "created if they are referenced in the AX program but do not exist on the "
+                    "input geometry.")
+            .setDefault(true)
+            .setCB([](bool& v, const char* arg) {
+                if (std::strcmp(arg, "ON") == 0)       v = true;
+                else if (std::strcmp(arg, "OFF") == 0) v = false;
+                else OPENVDB_THROW(CLIError, "Invalid option passed to --create-missing: '" << arg << "'");
+            })
+            .get();
+
+    ParamT<std::string> mGroup =
+        ParamBuilderT<std::string>()
+            .addOpt("--group [name]")
+            .setDoc("a point group to process. Only points that belong to this group are "
+                    "processed by the AX program. Note that this is equivalent to using:\n "
+                    "  if (!ingroup(\"name\")) return;\n"
+                    "at the start of the AX program.")
+            .setDefault("")
+            .get();
+
+    ParamT<size_t> mGrainSize =
+        ParamBuilderT<size_t>()
+            .addOpt("--points-grain [n]")
+            .setDoc("the threading grain size for processing nodes (Default: 1). A value of 0 "
+                    "disables threading.")
+            .setDefault(1)
+            .get();
+
+    ParamT<AttributeBindings> mBindings =
+        ParamBuilderT<AttributeBindings>()
+            .addOpt("--bindings [\"ax_name:point_attr_name,...\"]")
+            .setDoc("attribute bindings for points. The argument accepts a quoted string list of "
+                    "AX (source code) name to data (vdb attribute) name pairs joined by colons and "
+                    "seperated by commas. For example:\n"
+                    "  --bindings \"velocity:v,density:s\"\n"
+                    "binds velocity AX accesses to a 'v' attribute and density AX accesses to a 's' "
+                    "attribute. The following snippet would then alias these attributes:\n"
+                    "  v@velocity *= 5;   // actually accesses 'v' points\n"
+                    "   @density += 1.0f; // actually accesses 's' points")
+            .setDefault(AttributeBindings{})
+            .setCB([](AttributeBindings& bindings, const char* c) {
+                std::string source, target;
+                std::string* active = &source, *other = &target;
+                while (*c != '\0') {
+                    if (*c == ':') std::swap(active, other);
+                    else if (*c == ',') {
+                        std::swap(active, other);
+                        if (source.empty() || target.empty()) {
+                            OPENVDB_THROW(CLIError, "invalid string passed to --bindings. '" << c << "'");
+                        }
+                        bindings.set(source, target);
+                        source.clear();
+                        target.clear();
+                    }
+                    else {
+                        active->push_back(*c);
+                    }
+                    ++c;
+                }
+
+                if (source.empty() || target.empty()) {
+                    OPENVDB_THROW(CLIError, "invalid string passed to --bindings: '" << c << "'");
+                }
+                bindings.set(source, target);
+            })
+            .get();
+
+    cli::BasicParam<bool> mPostDelete = false;
 };
 
 namespace {
@@ -50,7 +159,7 @@ using KernelBufferRangeFunctionPtr = std::add_pointer<codegen::PointKernelBuffer
 using PointLeafLocalData = codegen::codegen_internal::PointLeafLocalData;
 
 #ifndef NDEBUG
-inline bool supported(const ast::tokens::CoreType type)
+[[maybe_unused]] inline bool supported(const ast::tokens::CoreType type)
 {
     switch (type) {
         case ast::tokens::BOOL    : return true;
@@ -223,7 +332,7 @@ struct PointFunctionArguments
         using FunctionTraitsT = codegen::PointKernelBufferRange::FunctionTraitsT;
         using ReturnT = FunctionTraitsT::ReturnType;
 
-        assert(mData.mUseBufferKernel);
+        OPENVDB_ASSERT(mData.mUseBufferKernel);
 
         return [&](const openvdb::Coord& origin, void* buffer, Index64* mask, const size_t size) -> ReturnT {
             mData.mKernelBufferRange(static_cast<FunctionTraitsT::Arg<0>::Type>(mData.mCustomData),
@@ -250,7 +359,6 @@ struct PointFunctionArguments
 
         // @todo  if the array is shared we should probably make it unique?
 
-#if OPENVDB_ABI_VERSION_NUMBER >= 9
         if (mData.mUseBufferKernel) {
             const_cast<points::AttributeArray&>(array).loadData();
             const char* data = array.constDataAsByteArray();
@@ -265,12 +373,6 @@ struct PointFunctionArguments
             mHandlesOrBuffers.emplace_back(handle->mHandle.get());
             mAttributeHandles.emplace_back(std::move(handle));
         }
-#else
-        assert(!mData.mUseBufferKernel);
-        typename ReadHandle<ValueT>::UniquePtr handle(new ReadHandle<ValueT>(leaf, Index(pos)));
-        mHandlesOrBuffers.emplace_back(handle->mHandle.get());
-        mAttributeHandles.emplace_back(std::move(handle));
-#endif
 
         mFlags.emplace_back(flag);
     }
@@ -282,7 +384,6 @@ struct PointFunctionArguments
         points::AttributeArray& array = leaf.attributeArray(pos);
         array.expand();
 
-#if OPENVDB_ABI_VERSION_NUMBER >= 9
         if (mData.mUseBufferKernel) {
             array.loadData();
             const char* data = array.constDataAsByteArray();
@@ -291,33 +392,27 @@ struct PointFunctionArguments
             const codegen::Codec* codec =
                 codegen::getCodec(ast::tokens::tokenFromTypeString(array.valueType()), array.codecType());
             if (codec) flag |= codec->flag();
-            assert(array.isDataLoaded() && !array.isUniform());
+            OPENVDB_ASSERT(array.isDataLoaded() && !array.isUniform());
         }
         else {
             typename WriteHandle<ValueT>::UniquePtr handle(new WriteHandle<ValueT>(leaf, Index(pos)));
             mHandlesOrBuffers.emplace_back(handle->mHandle.get());
             mAttributeHandles.emplace_back(std::move(handle));
         }
-#else
-        assert(!mData.mUseBufferKernel);
-        typename WriteHandle<ValueT>::UniquePtr handle(new WriteHandle<ValueT>(leaf, Index(pos)));
-        mHandlesOrBuffers.emplace_back(handle->mHandle.get());
-        mAttributeHandles.emplace_back(std::move(handle));
-#endif
 
         mFlags.emplace_back(flag);
     }
 
     inline void addGroupHandle(const LeafT& leaf, const std::string& name)
     {
-        assert(leaf.attributeSet().descriptor().hasGroup(name));
+        OPENVDB_ASSERT(leaf.attributeSet().descriptor().hasGroup(name));
         mGroupHandles.emplace_back(new points::GroupHandle(leaf.groupHandle(name)));
         mVoidGroupHandles.emplace_back(static_cast<void*>(mGroupHandles.back().get()));
     }
 
     inline void addGroupWriteHandle(LeafT& leaf, const std::string& name)
     {
-        assert(leaf.attributeSet().descriptor().hasGroup(name));
+        OPENVDB_ASSERT(leaf.attributeSet().descriptor().hasGroup(name));
         mGroupHandles.emplace_back(new points::GroupWriteHandle(leaf.groupWriteHandle(name)));
         mVoidGroupHandles.emplace_back(static_cast<void*>(mGroupHandles.back().get()));
     }
@@ -328,7 +423,7 @@ struct PointFunctionArguments
     addAttributeHandle(LeafT& leaf, const std::string& name, const ast::tokens::CoreType type, const bool write)
     {
         // assert so the executer can be marked as noexcept (assuming nothing throws in compute)
-        assert(supported(type) && "Could not retrieve attribute handle from unsupported type");
+        OPENVDB_ASSERT(supported(type) && "Could not retrieve attribute handle from unsupported type");
         switch (type) {
             case ast::tokens::BOOL    : return this->addAttributeHandleTyped<bool>(leaf, name, write);
             case ast::tokens::CHAR    : return this->addAttributeHandleTyped<char>(leaf, name, write);
@@ -363,7 +458,7 @@ private:
     {
         const size_t pos = leaf.attributeSet().find(name);
         //assert(!leaf.attributeSet().isShared(pos));
-        assert(pos != openvdb::points::AttributeSet::INVALID_POS);
+        OPENVDB_ASSERT(pos != openvdb::points::AttributeSet::INVALID_POS);
         if (write) this->addWriteHandle<ValueType>(leaf, pos);
         else       this->addHandle<ValueType>(leaf, pos);
     }
@@ -409,7 +504,7 @@ struct PointExecuterDeformer
     void apply(Vec3d& position, const IterT& iter) const
     {
         if (mFilter.valid(iter)) {
-            assert(mPws);
+            OPENVDB_ASSERT(mPws);
             position = Vec3d(mPws->get(*iter));
         }
     }
@@ -553,7 +648,7 @@ void processAttributes(points::PointDataGrid& grid,
                        Logger& logger)
 {
     const auto leafIter = grid.tree().cbeginLeaf();
-    assert(leafIter);
+    OPENVDB_ASSERT(leafIter);
 
     attributeInfo.reserve(registry.data().size());
 
@@ -588,7 +683,7 @@ void processAttributes(points::PointDataGrid& grid,
 
         if (pos != points::AttributeSet::INVALID_POS) {
             const points::AttributeArray* const array = leafIter->attributeSet().getConst(pos);
-            assert(array);
+            OPENVDB_ASSERT(array);
             if (array->stride() > 1) {
                 logger.warning("Attribute \"" + name + (name != iter.name() ? "\" [bound to \"" + iter.name() + "\"]" : "\"")
                     + " on grid \"" + grid.getName() + "\"is a strided (array) attribute. "
@@ -610,7 +705,7 @@ void processAttributes(points::PointDataGrid& grid,
             continue;
         }
 
-        assert(supported(iter.type()));
+        OPENVDB_ASSERT(supported(iter.type()));
         const NamePair type = typePairFromToken(iter.type());
         points::appendAttribute(grid.tree(), name, type);
     }
@@ -659,18 +754,18 @@ PointExecutable::PointExecutable(const std::shared_ptr<const llvm::LLVMContext>&
     , mAttributeRegistry(attributeRegistry)
     , mCustomData(customData)
     , mFunctionAddresses(functions)
-    , mSettings(new Settings)
+    , mSettings(new Settings<false>)
 {
-    assert(mContext);
-    assert(mExecutionEngine);
-    assert(mAttributeRegistry);
+    OPENVDB_ASSERT(mContext);
+    OPENVDB_ASSERT(mExecutionEngine);
+    OPENVDB_ASSERT(mAttributeRegistry);
 
     // parse the AST for known functions which require pre/post processing
     mSettings->mPostDelete = ast::callsFunction(ast, "deletepoint");
 
     // Set up the default attribute bindings
     for (const auto& iter : mAttributeRegistry->data()) {
-        mSettings->mBindings.set(iter.name(), iter.name());
+        mSettings->mBindings.get().set(iter.name(), iter.name());
     }
 }
 
@@ -680,8 +775,7 @@ PointExecutable::PointExecutable(const PointExecutable& other)
     , mAttributeRegistry(other.mAttributeRegistry)
     , mCustomData(other.mCustomData)
     , mFunctionAddresses(other.mFunctionAddresses)
-    , mSettings(new Settings(*other.mSettings)) {
-}
+    , mSettings(new Settings<false>(*other.mSettings)) {}
 
 PointExecutable::~PointExecutable() {}
 
@@ -712,7 +806,7 @@ void PointExecutable::execute(openvdb::points::PointDataGrid& grid) const
     // create any missing attributes and handle temp ws position storage
     data.mPositionAccess = {false, false};
     processAttributes(grid, data.mAttributeInfo, data.mPositionAccess, data.mPositionAttribute,
-                      *mAttributeRegistry, mSettings->mBindings,
+                      *mAttributeRegistry, mSettings->mBindings.get(),
                       mSettings->mCreateMissing, *logger);
 
     data.mKernelAttributeArray =
@@ -729,7 +823,7 @@ void PointExecutable::execute(openvdb::points::PointDataGrid& grid) const
 
     // init the internal dead group if necessary
 
-    if (mSettings->mPostDelete) {
+    if (mSettings->mPostDelete.get()) {
         // @todo  This group is hard coded in the deletepoint function call - we
         //  should perhaps instead pass in a unique group string to the point kernel
         if (leafIter->attributeSet().descriptor().hasGroup("__ax_dead")) {
@@ -742,15 +836,15 @@ void PointExecutable::execute(openvdb::points::PointDataGrid& grid) const
         }
     }
 
-    const bool usingGroup = !mSettings->mGroup.empty();
+    const bool usingGroup = !mSettings->mGroup.get().empty();
     if (usingGroup) {
-        if (!leafIter->attributeSet().descriptor().hasGroup(mSettings->mGroup)) {
-            logger->error("Requested point group \"" + mSettings->mGroup +
+        if (!leafIter->attributeSet().descriptor().hasGroup(mSettings->mGroup.get())) {
+            logger->error("Requested point group \"" + mSettings->mGroup.get() +
                 "\" on grid \"" + grid.getName() + "\" does not exist.");
         }
         else {
             data.mGroupIndex =
-                leafIter->attributeSet().groupIndex(mSettings->mGroup);
+                leafIter->attributeSet().groupIndex(mSettings->mGroup.get());
         }
     }
 
@@ -759,30 +853,25 @@ void PointExecutable::execute(openvdb::points::PointDataGrid& grid) const
     // Compute whether we can use the accelerated kernel
     // @note  Assumes attributes are valid (i.e. has errored out if they are not)
 
-#if OPENVDB_ABI_VERSION_NUMBER >= 9
     if (!usingGroup) {
         const auto& desc = leafIter->attributeSet().descriptor();
         data.mUseBufferKernel = checkCodecs(desc, *mAttributeRegistry,
-            mSettings->mBindings,
+            mSettings->mBindings.get(),
             data.mPositionAttribute);
     }
     else {
         // if a group has been specified we can't use the buffer range yet
         data.mUseBufferKernel = false;
     }
-#else
-    // can't access data buffers until ABI >= 9
-    data.mUseBufferKernel = false;
-#endif
 
     // execute
 
     LeafManagerT leafManager(grid.tree());
     std::vector<PointLeafLocalData::UniquePtr> leafLocalData(leafManager.leafCount());
-    const bool threaded = mSettings->mGrainSize > 0;
+    const bool threaded = mSettings->mGrainSize.get() > 0;
 
     PointExecuterOp executerOp(data, leafLocalData);
-    leafManager.foreach(executerOp, threaded, mSettings->mGrainSize);
+    leafManager.foreach(executerOp, threaded, mSettings->mGrainSize.get());
 
     // Check to see if any new data has been added and apply it accordingly
 
@@ -868,7 +957,7 @@ void PointExecutable::execute(openvdb::points::PointDataGrid& grid) const
         points::dropAttribute(grid.tree(), data.mPositionAttribute);
     }
 
-    if (mSettings->mPostDelete) {
+    if (mSettings->mPostDelete.get()) {
         points::deleteFromGroup(grid.tree(), "__ax_dead", false, /*drop=*/true);
     }
 }
@@ -922,12 +1011,12 @@ void PointExecutable::setAttributeBindings(const AttributeBindings& bindings)
 {
     //@todo: warn when inferred P, Cd, N etc are bound they default to vec3f
     for (const auto& binding : bindings.axToDataMap()) {
-        mSettings->mBindings.set(binding.first, binding.second);
+        mSettings->mBindings.get().set(binding.first, binding.second);
     }
 
     // check the registry to make sure everything is bound
     for (const auto& access : mAttributeRegistry->data()) {
-        if (!mSettings->mBindings.isBoundAXName(access.name())) {
+        if (!mSettings->mBindings.get().isBoundAXName(access.name())) {
             if (bindings.isBoundDataName(access.name())) {
                 OPENVDB_THROW(AXExecutionError, "AX attribute \"@"
                     + access.name() + "\" not bound to any point attribute."
@@ -937,7 +1026,7 @@ void PointExecutable::setAttributeBindings(const AttributeBindings& bindings)
             else {
                 // rebind to itself as it may have been unbound
                 // by a previous set call
-                mSettings->mBindings.set(access.name(), access.name());
+                mSettings->mBindings.get().set(access.name(), access.name());
             }
         }
     }
@@ -945,7 +1034,7 @@ void PointExecutable::setAttributeBindings(const AttributeBindings& bindings)
 
 const AttributeBindings& PointExecutable::getAttributeBindings() const
 {
-    return mSettings->mBindings;
+    return mSettings->mBindings.get();
 }
 
 //
@@ -964,7 +1053,7 @@ bool PointExecutable::usesAcceleratedKernel(const points::PointDataTree& tree) c
 
     for (const auto& iter : mAttributeRegistry->data()) {
         // get the corresponding point attributes
-        const std::string* nameptr = mSettings->mBindings.dataNameBoundTo(iter.name());
+        const std::string* nameptr = mSettings->mBindings.get().dataNameBoundTo(iter.name());
         if (!nameptr) continue;
         const std::string& name = *nameptr;
         if (points::AttributeSet::INVALID_POS == desc->find(name)) {
@@ -980,7 +1069,44 @@ bool PointExecutable::usesAcceleratedKernel(const points::PointDataTree& tree) c
     desc = desc->duplicateAppend("P", typePairFromToken(ast::tokens::CoreType::VEC3F));
 
     return checkCodecs(*desc, *mAttributeRegistry,
-        mSettings->mBindings, "P");
+        mSettings->mBindings.get(), "P");
+}
+
+
+////////////////////////////////////////////////////////
+////////////////////////////////////////////////////////
+
+
+PointExecutable::CLI::CLI()
+    : mSettings(new PointExecutable::Settings<true>) {}
+PointExecutable::CLI::~CLI() {}
+PointExecutable::CLI::CLI(CLI&& other) {
+    mSettings = std::move(other.mSettings);
+}
+PointExecutable::CLI& PointExecutable::CLI::operator=(CLI&& other) {
+    mSettings = std::move(other.mSettings);
+    return *this;
+}
+
+PointExecutable::CLI
+PointExecutable::CLI::create(size_t argc, const char* argv[], bool* flags)
+{
+    CLI cli;
+    openvdb::ax::cli::init(argc, argv, {}, cli.mSettings->optional(), flags);
+    return cli;
+}
+
+void PointExecutable::CLI::usage(std::ostream& os, const bool verbose)
+{
+    PointExecutable::Settings<true> S;
+    for (const auto& P : S.optional()) {
+        ax::cli::usage(os, P->opts(), P->doc(), verbose);
+    }
+}
+
+void PointExecutable::setSettingsFromCLI(const PointExecutable::CLI& cli)
+{
+    mSettings->init(*cli.mSettings);
 }
 
 
