@@ -1,5 +1,5 @@
 // Copyright Contributors to the OpenVDB Project
-// SPDX-License-Identifier: MPL-2.0
+// SPDX-License-Identifier: Apache-2.0
 
 /// @file compiler/PointExecutable.h
 ///
@@ -140,7 +140,30 @@ public:
     ////////////////////////////////////////////////////////
 
     // foward declaration of settings for this executable
-    struct Settings;
+    template <bool> struct Settings;
+
+    /// @brief Command Line Interface handling for the PointExecutable.
+    /// @details  This class wraps the logic for converting commands specific
+    ///   to the PointExecutable to the internal Settings. Subsequent
+    ///   executables can be initialized from the CLI object that gets created
+    struct OPENVDB_AX_API CLI
+    {
+        ~CLI();
+        CLI(CLI&&);
+        CLI& operator=(CLI&&);
+        static CLI create(size_t argc, const char* argv[], bool* used=nullptr);
+        static void usage(std::ostream& os, const bool verbose);
+    private:
+        friend class PointExecutable;
+        CLI();
+        std::unique_ptr<Settings<true>> mSettings;
+    };
+
+    /// @brief  Intialize the Settings of this executables from the CLI object
+    /// @param cli The CLI object
+    void setSettingsFromCLI(const CLI& cli);
+
+    ////////////////////////////////////////////////////////
 
 private:
     friend class Compiler;
@@ -178,7 +201,7 @@ private:
     const AttributeRegistry::ConstPtr mAttributeRegistry;
     const CustomData::ConstPtr mCustomData;
     const std::unordered_map<std::string, uint64_t> mFunctionAddresses;
-    std::unique_ptr<Settings> mSettings;
+    std::unique_ptr<Settings<false>> mSettings;
 };
 
 } // namespace ax
