@@ -1,5 +1,5 @@
 // Copyright Contributors to the OpenVDB Project
-// SPDX-License-Identifier: MPL-2.0
+// SPDX-License-Identifier: Apache-2.0
 
 #include <openvdb_ax/compiler/Compiler.h>
 #include <openvdb_ax/compiler/VolumeExecutable.h>
@@ -13,6 +13,12 @@ class TestVolumeExecutable : public CppUnit::TestCase
 {
 public:
 
+#if OPENVDB_ABI_VERSION_NUMBER >= 12
+    using LeafIndexType = openvdb::Index64;
+#else
+    using LeafIndexType = openvdb::Index32;
+#endif
+
     CPPUNIT_TEST_SUITE(TestVolumeExecutable);
     CPPUNIT_TEST(testConstructionDestruction);
     CPPUNIT_TEST(testCreateMissingGrids);
@@ -20,6 +26,7 @@ public:
     CPPUNIT_TEST(testActiveTileStreaming);
     CPPUNIT_TEST(testCompilerCases);
     CPPUNIT_TEST(testExecuteBindings);
+    CPPUNIT_TEST(testCLI);
     CPPUNIT_TEST_SUITE_END();
 
     void testConstructionDestruction();
@@ -28,6 +35,7 @@ public:
     void testActiveTileStreaming();
     void testCompilerCases();
     void testExecuteBindings();
+    void testCLI();
 };
 
 CPPUNIT_TEST_SUITE_REGISTRATION(TestVolumeExecutable);
@@ -45,6 +53,12 @@ TestVolumeExecutable::testConstructionDestruction()
     CPPUNIT_ASSERT(openvdb::ax::isInitialized());
 
     std::shared_ptr<llvm::LLVMContext> C(new llvm::LLVMContext);
+#if LLVM_VERSION_MAJOR >= 15
+    // This will not work from LLVM 16. We'll need to fix this
+    // https://llvm.org/docs/OpaquePointers.html
+    C->setOpaquePointers(false);
+#endif
+
     std::unique_ptr<llvm::Module> M(new llvm::Module("test_module", *C));
     std::shared_ptr<const llvm::ExecutionEngine> E(llvm::EngineBuilder(std::move(M))
             .setEngineKind(llvm::EngineKind::JIT)
@@ -130,9 +144,9 @@ TestVolumeExecutable::testTreeExecutionLevel()
     CPPUNIT_ASSERT(openvdb::ax::VolumeExecutable::Streaming::OFF ==
         executable->getActiveTileStreaming());
 
-    using NodeT0 = openvdb::FloatGrid::Accessor::NodeT0;
-    using NodeT1 = openvdb::FloatGrid::Accessor::NodeT1;
-    using NodeT2 = openvdb::FloatGrid::Accessor::NodeT2;
+    using NodeT0 = openvdb::FloatGrid::Accessor::template NodeTypeAtLevel<0>;
+    using NodeT1 = openvdb::FloatGrid::Accessor::template NodeTypeAtLevel<1>;
+    using NodeT2 = openvdb::FloatGrid::Accessor::template NodeTypeAtLevel<2>;
 
     openvdb::FloatGrid grid;
     grid.setName("test");
@@ -147,7 +161,7 @@ TestVolumeExecutable::testTreeExecutionLevel()
     const openvdb::FloatTree copy = tree;
     // check config
     auto CHECK_CONFIG = [&]() {
-        CPPUNIT_ASSERT_EQUAL(openvdb::Index32(1), tree.leafCount());
+        CPPUNIT_ASSERT_EQUAL(LeafIndexType(1), tree.leafCount());
         CPPUNIT_ASSERT_EQUAL(openvdb::Index64(3), tree.activeTileCount());
         CPPUNIT_ASSERT_EQUAL(int(openvdb::FloatTree::DEPTH-4), tree.getValueDepth(openvdb::Coord(0)));
         CPPUNIT_ASSERT_EQUAL(int(openvdb::FloatTree::DEPTH-3), tree.getValueDepth(openvdb::Coord(NodeT2::DIM)));
@@ -314,9 +328,9 @@ TestVolumeExecutable::testTreeExecutionLevel()
 void
 TestVolumeExecutable::testActiveTileStreaming()
 {
-    using NodeT0 = openvdb::FloatGrid::Accessor::NodeT0;
-    using NodeT1 = openvdb::FloatGrid::Accessor::NodeT1;
-    using NodeT2 = openvdb::FloatGrid::Accessor::NodeT2;
+    using NodeT0 = openvdb::FloatGrid::Accessor::template NodeTypeAtLevel<0>;
+    using NodeT1 = openvdb::FloatGrid::Accessor::template NodeTypeAtLevel<1>;
+    using NodeT2 = openvdb::FloatGrid::Accessor::template NodeTypeAtLevel<2>;
 
     //
 
@@ -349,7 +363,7 @@ TestVolumeExecutable::testActiveTileStreaming()
         CPPUNIT_ASSERT_EQUAL(openvdb::Index(openvdb::FloatTree::DEPTH-1), max);
         executable->execute(grid);
 
-        CPPUNIT_ASSERT_EQUAL(openvdb::Index32(1), tree.leafCount());
+        CPPUNIT_ASSERT_EQUAL(LeafIndexType(1), tree.leafCount());
         CPPUNIT_ASSERT_EQUAL(openvdb::Index64(3), tree.activeTileCount());
         CPPUNIT_ASSERT_EQUAL(int(openvdb::FloatTree::DEPTH-4), tree.getValueDepth(openvdb::Coord(0)));
         CPPUNIT_ASSERT_EQUAL(int(openvdb::FloatTree::DEPTH-3), tree.getValueDepth(openvdb::Coord(NodeT2::DIM)));
@@ -396,7 +410,7 @@ TestVolumeExecutable::testActiveTileStreaming()
             openvdb::Index64(NodeT1::NUM_VOXELS) +
             openvdb::Index64(NodeT0::NUM_VOXELS);
 
-        CPPUNIT_ASSERT_EQUAL(openvdb::Index32(voxels / openvdb::FloatTree::LeafNodeType::NUM_VOXELS), tree.leafCount());
+        CPPUNIT_ASSERT_EQUAL(LeafIndexType(voxels / openvdb::FloatTree::LeafNodeType::NUM_VOXELS), tree.leafCount());
         CPPUNIT_ASSERT_EQUAL(openvdb::Index64(0), tree.activeTileCount());
         CPPUNIT_ASSERT_EQUAL(int(openvdb::FloatTree::DEPTH-1), tree.getValueDepth(openvdb::Coord(0)));
         CPPUNIT_ASSERT_EQUAL(int(openvdb::FloatTree::DEPTH-1), tree.getValueDepth(openvdb::Coord(NodeT1::DIM)));
@@ -455,7 +469,7 @@ TestVolumeExecutable::testActiveTileStreaming()
             ((n1ChildCount * (n2ChildAxisCount * n2ChildAxisCount)) - leafs) // NodeT1 face tiles (NodeT0) - leafs
             + 1 /*NodeT1*/ + 1 /*NodeT0*/;
 
-        CPPUNIT_ASSERT_EQUAL(openvdb::Index32(leafs), tree.leafCount());
+        CPPUNIT_ASSERT_EQUAL(LeafIndexType(leafs), tree.leafCount());
         CPPUNIT_ASSERT_EQUAL(openvdb::Index64(tiles), tree.activeTileCount());
         CPPUNIT_ASSERT_EQUAL(int(openvdb::FloatTree::DEPTH-3), tree.getValueDepth(openvdb::Coord(NodeT2::DIM)));
         CPPUNIT_ASSERT_EQUAL(int(openvdb::FloatTree::DEPTH-2), tree.getValueDepth(openvdb::Coord(NodeT2::DIM+NodeT1::DIM)));
@@ -504,7 +518,7 @@ TestVolumeExecutable::testActiveTileStreaming()
 
         executable->execute(grid);
 
-        CPPUNIT_ASSERT_EQUAL(openvdb::Index32(0), tree.leafCount());
+        CPPUNIT_ASSERT_EQUAL(LeafIndexType(0), tree.leafCount());
         CPPUNIT_ASSERT_EQUAL(openvdb::Index64(5), tree.activeTileCount());
         CPPUNIT_ASSERT_EQUAL(int(openvdb::FloatTree::DEPTH-3), tree.getValueDepth(openvdb::Coord(NodeT1::DIM*0, 0, 0)));
         CPPUNIT_ASSERT_EQUAL(int(openvdb::FloatTree::DEPTH-3), tree.getValueDepth(openvdb::Coord(NodeT1::DIM*1, 0, 0)));
@@ -559,7 +573,7 @@ TestVolumeExecutable::testActiveTileStreaming()
             (n1ChildCount - leafs) // NodeT1 face tiles (NodeT0) - leafs
             + 3 /*NodeT1*/ + 1 /*NodeT0*/;
 
-        CPPUNIT_ASSERT_EQUAL(openvdb::Index32(leafs), tree.leafCount());
+        CPPUNIT_ASSERT_EQUAL(LeafIndexType(leafs), tree.leafCount());
         CPPUNIT_ASSERT_EQUAL(openvdb::Index64(tiles), tree.activeTileCount());
         CPPUNIT_ASSERT_EQUAL(int(openvdb::BoolTree::DEPTH-3), tree.getValueDepth(openvdb::Coord(NodeT1::DIM*1, 0, 0)));
         CPPUNIT_ASSERT_EQUAL(int(openvdb::BoolTree::DEPTH-3), tree.getValueDepth(openvdb::Coord(NodeT1::DIM*2, 0, 0)));
@@ -576,11 +590,13 @@ TestVolumeExecutable::testActiveTileStreaming()
     }
 
     // test spatially varying voxelization for string grids which use specialized implementations
+    // Note: StringGrids are no longer registered by default
     {
-OPENVDB_NO_DEPRECATION_WARNING_BEGIN
-        openvdb::StringGrid grid;
+        using StringTree = openvdb::tree::Tree4<std::string, 5, 4, 3>::Type;
+        using StringGrid = openvdb::Grid<openvdb::tree::Tree4<std::string, 5, 4, 3>::Type>;
+        StringGrid grid;
         grid.setName("test");
-        openvdb::StringTree& tree = grid.tree();
+        StringTree& tree = grid.tree();
         tree.addTile(2, openvdb::Coord(NodeT1::DIM*0, 0, 0), "foo", /*active*/true); // NodeT1 tile
         tree.addTile(2, openvdb::Coord(NodeT1::DIM*1, 0, 0), "foo", /*active*/true); // NodeT1 tile
         tree.addTile(2, openvdb::Coord(NodeT1::DIM*2, 0, 0), "foo", /*active*/true); // NodeT1 tile
@@ -597,14 +613,14 @@ OPENVDB_NO_DEPRECATION_WARNING_BEGIN
             executable->getActiveTileStreaming("empty", openvdb::ax::ast::tokens::CoreType::FLOAT));
         executable->getTreeExecutionLevel(min,max);
         CPPUNIT_ASSERT_EQUAL(openvdb::Index(0), min);
-        CPPUNIT_ASSERT_EQUAL(openvdb::Index(openvdb::StringTree::DEPTH-1), max);
+        CPPUNIT_ASSERT_EQUAL(openvdb::Index(StringTree::DEPTH-1), max);
 
         executable->execute(grid);
 
         const openvdb::Index64 face = NodeT1::DIM * NodeT1::DIM; // face voxel count of NodeT2 x==0
         const openvdb::Index64 leafs = // expected leaf nodes that need to be created
-            (face * openvdb::StringTree::LeafNodeType::DIM) /
-            openvdb::StringTree::LeafNodeType::NUM_VOXELS;
+            (face * StringTree::LeafNodeType::DIM) /
+            StringTree::LeafNodeType::NUM_VOXELS;
 
         // number of child nodes in NodeT1;
         const openvdb::Index64 n1ChildAxisCount = NodeT1::DIM / NodeT1::getChildDim();
@@ -614,12 +630,12 @@ OPENVDB_NO_DEPRECATION_WARNING_BEGIN
             (n1ChildCount - leafs) // NodeT1 face tiles (NodeT0) - leafs
             + 3 /*NodeT1*/ + 1 /*NodeT0*/;
 
-        CPPUNIT_ASSERT_EQUAL(openvdb::Index32(leafs), tree.leafCount());
+        CPPUNIT_ASSERT_EQUAL(LeafIndexType(leafs), tree.leafCount());
         CPPUNIT_ASSERT_EQUAL(openvdb::Index64(tiles), tree.activeTileCount());
-        CPPUNIT_ASSERT_EQUAL(int(openvdb::StringTree::DEPTH-3), tree.getValueDepth(openvdb::Coord(NodeT1::DIM*1, 0, 0)));
-        CPPUNIT_ASSERT_EQUAL(int(openvdb::StringTree::DEPTH-3), tree.getValueDepth(openvdb::Coord(NodeT1::DIM*2, 0, 0)));
-        CPPUNIT_ASSERT_EQUAL(int(openvdb::StringTree::DEPTH-3), tree.getValueDepth(openvdb::Coord(NodeT1::DIM*3, 0, 0)));
-        CPPUNIT_ASSERT_EQUAL(int(openvdb::StringTree::DEPTH-2), tree.getValueDepth(openvdb::Coord(NodeT2::DIM)));
+        CPPUNIT_ASSERT_EQUAL(int(StringTree::DEPTH-3), tree.getValueDepth(openvdb::Coord(NodeT1::DIM*1, 0, 0)));
+        CPPUNIT_ASSERT_EQUAL(int(StringTree::DEPTH-3), tree.getValueDepth(openvdb::Coord(NodeT1::DIM*2, 0, 0)));
+        CPPUNIT_ASSERT_EQUAL(int(StringTree::DEPTH-3), tree.getValueDepth(openvdb::Coord(NodeT1::DIM*3, 0, 0)));
+        CPPUNIT_ASSERT_EQUAL(int(StringTree::DEPTH-2), tree.getValueDepth(openvdb::Coord(NodeT2::DIM)));
         CPPUNIT_ASSERT_EQUAL((openvdb::Index64(NodeT1::NUM_VOXELS)*4) +
             openvdb::Index64(NodeT0::NUM_VOXELS), tree.activeVoxelCount());
 
@@ -628,7 +644,6 @@ OPENVDB_NO_DEPRECATION_WARNING_BEGIN
             if (coord.x() == 0) CPPUNIT_ASSERT_EQUAL(*it, std::string("bar"));
             else                CPPUNIT_ASSERT_EQUAL(*it, std::string("foo"));
         });
-OPENVDB_NO_DEPRECATION_WARNING_END
     }
 
     // test streaming with an OFF iterator (no streaming behaviour) and an ALL iterator (streaming behaviour for ON values only)
@@ -659,7 +674,7 @@ OPENVDB_NO_DEPRECATION_WARNING_END
 
         executable->execute(grid);
 
-        CPPUNIT_ASSERT_EQUAL(openvdb::Index32(1), tree.leafCount());
+        CPPUNIT_ASSERT_EQUAL(LeafIndexType(1), tree.leafCount());
         CPPUNIT_ASSERT_EQUAL(openvdb::Index64(2), tree.activeTileCount());
         CPPUNIT_ASSERT(tree.hasSameTopology(copy));
         CPPUNIT_ASSERT_EQUAL(int(openvdb::FloatTree::DEPTH-3), tree.getValueDepth(openvdb::Coord(0)));
@@ -706,7 +721,7 @@ OPENVDB_NO_DEPRECATION_WARNING_END
             openvdb::Index64(NodeT1::NUM_VOXELS) +
             openvdb::Index64(NodeT0::NUM_VOXELS);
 
-        CPPUNIT_ASSERT_EQUAL(openvdb::Index32(voxels / openvdb::FloatTree::LeafNodeType::NUM_VOXELS) + 1, tree.leafCount());
+        CPPUNIT_ASSERT_EQUAL(LeafIndexType(voxels / openvdb::FloatTree::LeafNodeType::NUM_VOXELS) + 1, tree.leafCount());
         CPPUNIT_ASSERT_EQUAL(openvdb::Index64(0), tree.activeTileCount());
         CPPUNIT_ASSERT_EQUAL(voxels, tree.activeVoxelCount());
         CPPUNIT_ASSERT_EQUAL(leaf, tree.probeLeaf(openvdb::Coord(NodeT1::DIM + NodeT0::DIM)));
@@ -1002,3 +1017,282 @@ TestVolumeExecutable::testExecuteBindings()
         CPPUNIT_ASSERT_EQUAL(*bindingsOnExecutable.dataNameBoundTo("c"), std::string("c"));
     }
 }
+
+void
+TestVolumeExecutable::testCLI()
+{
+    using namespace openvdb;
+    using CLI = openvdb::ax::VolumeExecutable::CLI;
+
+    struct UnusedCLIParam : public openvdb::Exception {
+        UnusedCLIParam() noexcept: Exception( "UnusedCLIParam" ) {} \
+        explicit UnusedCLIParam(const std::string& msg) noexcept: Exception( "UnusedCLIParam" , &msg) {}
+    };
+
+    auto CreateCLI = [](const char* c, bool throwIfUnused = true)
+    {
+        std::vector<std::string> strs;
+        const char* s = c;
+        while (*c != '\0') {
+            if (*c == ' ') {
+                strs.emplace_back(std::string(s, c-s));
+                ++c;
+                s = c;
+            }
+            else {
+                ++c;
+            }
+        }
+        if (*s != '\0') strs.emplace_back(std::string(s, c-s));
+
+        std::vector<const char*> args;
+        for (auto& str : strs) args.emplace_back(str.c_str());
+
+        std::unique_ptr<bool[]> flags(new bool[args.size()]);
+        std::fill(flags.get(), flags.get()+args.size(), false);
+
+        auto cli = CLI::create(args.size(), args.data(), flags.get());
+        if (throwIfUnused) {
+            for (size_t i = 0; i < args.size(); ++i) {
+                if (!flags[i]) OPENVDB_THROW(UnusedCLIParam, "unused param");
+            }
+        }
+        return cli;
+    };
+
+    ax::Compiler::UniquePtr compiler = ax::Compiler::create();
+
+    auto defaultExe = compiler->compile<openvdb::ax::VolumeExecutable>("");
+    Index defaultMinLevel, defaultMaxLevel;
+    defaultExe->getTreeExecutionLevel(defaultMinLevel, defaultMaxLevel);
+    const auto defaultCreateMissing = defaultExe->getCreateMissing();
+    const auto defaultTileStream = defaultExe->getActiveTileStreaming();
+    const auto defaultValueIter = defaultExe->getValueIterator();
+    const auto defaultGrain = defaultExe->getGrainSize();
+    const auto defaultTileGrain = defaultExe->getActiveTileStreamingGrainSize();
+    const auto defaultBindings = defaultExe->getAttributeBindings();
+
+    CPPUNIT_ASSERT_THROW(CreateCLI("--unknown"), UnusedCLIParam);
+    CPPUNIT_ASSERT_THROW(CreateCLI("-unknown"), UnusedCLIParam);
+    CPPUNIT_ASSERT_THROW(CreateCLI("-"), UnusedCLIParam);
+    CPPUNIT_ASSERT_THROW(CreateCLI("--"), UnusedCLIParam);
+    CPPUNIT_ASSERT_THROW(CreateCLI("-- "), UnusedCLIParam);
+
+    {
+        CLI cli = CreateCLI("");
+        auto exe = compiler->compile<openvdb::ax::VolumeExecutable>("");
+        CPPUNIT_ASSERT_NO_THROW(exe->setSettingsFromCLI(cli));
+
+        Index min,max;
+        exe->getTreeExecutionLevel(min, max);
+        CPPUNIT_ASSERT_EQUAL(defaultMinLevel, min);
+        CPPUNIT_ASSERT_EQUAL(defaultMaxLevel, max);
+        CPPUNIT_ASSERT_EQUAL(defaultCreateMissing, exe->getCreateMissing());
+        CPPUNIT_ASSERT_EQUAL(defaultTileStream, exe->getActiveTileStreaming());
+        CPPUNIT_ASSERT_EQUAL(defaultValueIter, exe->getValueIterator());
+        CPPUNIT_ASSERT_EQUAL(defaultGrain, exe->getGrainSize());
+        CPPUNIT_ASSERT_EQUAL(defaultTileGrain, exe->getActiveTileStreamingGrainSize());
+        CPPUNIT_ASSERT_EQUAL(defaultBindings, exe->getAttributeBindings());
+    }
+
+    // --create-missing
+    {
+        CPPUNIT_ASSERT_THROW(CreateCLI("--create-missing"), openvdb::CLIError);
+        CPPUNIT_ASSERT_THROW(CreateCLI("--create-missing invalid"), openvdb::CLIError);
+        CPPUNIT_ASSERT_THROW(CreateCLI("--create-missing --group test"), openvdb::CLIError);
+
+        CLI cli = CreateCLI("--create-missing ON");
+        auto exe = compiler->compile<openvdb::ax::VolumeExecutable>("");
+        CPPUNIT_ASSERT_NO_THROW(exe->setSettingsFromCLI(cli));
+
+        Index min,max;
+        exe->getTreeExecutionLevel(min, max);
+        CPPUNIT_ASSERT_EQUAL(defaultMinLevel, min);
+        CPPUNIT_ASSERT_EQUAL(defaultMaxLevel, max);
+        CPPUNIT_ASSERT_EQUAL(true, exe->getCreateMissing());
+        CPPUNIT_ASSERT_EQUAL(defaultTileStream, exe->getActiveTileStreaming());
+        CPPUNIT_ASSERT_EQUAL(defaultValueIter, exe->getValueIterator());
+        CPPUNIT_ASSERT_EQUAL(defaultGrain, exe->getGrainSize());
+        CPPUNIT_ASSERT_EQUAL(defaultTileGrain, exe->getActiveTileStreamingGrainSize());
+        CPPUNIT_ASSERT_EQUAL(defaultBindings, exe->getAttributeBindings());
+    }
+
+    // --tile-stream
+    {
+        CPPUNIT_ASSERT_THROW(CreateCLI("--tile-stream"), openvdb::CLIError);
+        CPPUNIT_ASSERT_THROW(CreateCLI("--tile-stream invalid"), openvdb::CLIError);
+        CPPUNIT_ASSERT_THROW(CreateCLI("--tile-stream --group test"), openvdb::CLIError);
+
+        CLI cli = CreateCLI("--tile-stream ON");
+        auto exe = compiler->compile<openvdb::ax::VolumeExecutable>("");
+        CPPUNIT_ASSERT_NO_THROW(exe->setSettingsFromCLI(cli));
+
+        Index min,max;
+        exe->getTreeExecutionLevel(min, max);
+        CPPUNIT_ASSERT_EQUAL(defaultMinLevel, min);
+        CPPUNIT_ASSERT_EQUAL(defaultMaxLevel, max);
+        CPPUNIT_ASSERT_EQUAL(defaultCreateMissing, exe->getCreateMissing());
+        CPPUNIT_ASSERT_EQUAL(openvdb::ax::VolumeExecutable::Streaming::ON, exe->getActiveTileStreaming());
+        CPPUNIT_ASSERT_EQUAL(defaultValueIter, exe->getValueIterator());
+        CPPUNIT_ASSERT_EQUAL(defaultGrain, exe->getGrainSize());
+        CPPUNIT_ASSERT_EQUAL(defaultTileGrain, exe->getActiveTileStreamingGrainSize());
+        CPPUNIT_ASSERT_EQUAL(defaultBindings, exe->getAttributeBindings());
+    }
+
+    // --node-iter
+    {
+        CPPUNIT_ASSERT_THROW(CreateCLI("--node-iter"), openvdb::CLIError);
+        CPPUNIT_ASSERT_THROW(CreateCLI("--node-iter invalid"), openvdb::CLIError);
+        CPPUNIT_ASSERT_THROW(CreateCLI("--node-iter --group test"), openvdb::CLIError);
+
+        CLI cli = CreateCLI("--node-iter ALL");
+        auto exe = compiler->compile<openvdb::ax::VolumeExecutable>("");
+        CPPUNIT_ASSERT_NO_THROW(exe->setSettingsFromCLI(cli));
+
+        Index min,max;
+        exe->getTreeExecutionLevel(min, max);
+        CPPUNIT_ASSERT_EQUAL(defaultMinLevel, min);
+        CPPUNIT_ASSERT_EQUAL(defaultMaxLevel, max);
+        CPPUNIT_ASSERT_EQUAL(defaultCreateMissing, exe->getCreateMissing());
+        CPPUNIT_ASSERT_EQUAL(defaultTileStream, exe->getActiveTileStreaming());
+        CPPUNIT_ASSERT_EQUAL(openvdb::ax::VolumeExecutable::IterType::ALL, exe->getValueIterator());
+        CPPUNIT_ASSERT_EQUAL(defaultGrain, exe->getGrainSize());
+        CPPUNIT_ASSERT_EQUAL(defaultTileGrain, exe->getActiveTileStreamingGrainSize());
+        CPPUNIT_ASSERT_EQUAL(defaultBindings, exe->getAttributeBindings());
+    }
+
+    // --tree-level
+    {
+        CPPUNIT_ASSERT_THROW(CreateCLI("--tree-level"), openvdb::CLIError);
+        CPPUNIT_ASSERT_THROW(CreateCLI("--tree-level invalid"), openvdb::CLIError);
+        CPPUNIT_ASSERT_THROW(CreateCLI("--tree-level --group test"), openvdb::CLIError);
+
+        CLI cli = CreateCLI("--tree-level 0");
+        auto exe = compiler->compile<openvdb::ax::VolumeExecutable>("");
+        CPPUNIT_ASSERT_NO_THROW(exe->setSettingsFromCLI(cli));
+
+        Index min,max;
+        exe->getTreeExecutionLevel(min, max);
+        CPPUNIT_ASSERT_EQUAL(min, Index(0));
+        CPPUNIT_ASSERT_EQUAL(defaultMaxLevel, max);
+        CPPUNIT_ASSERT_EQUAL(defaultCreateMissing, exe->getCreateMissing());
+        CPPUNIT_ASSERT_EQUAL(defaultTileStream, exe->getActiveTileStreaming());
+        CPPUNIT_ASSERT_EQUAL(defaultValueIter, exe->getValueIterator());
+        CPPUNIT_ASSERT_EQUAL(defaultGrain, exe->getGrainSize());
+        CPPUNIT_ASSERT_EQUAL(defaultTileGrain, exe->getActiveTileStreamingGrainSize());
+        CPPUNIT_ASSERT_EQUAL(defaultBindings, exe->getAttributeBindings());
+
+        cli = CreateCLI("--tree-level 1:2");
+        CPPUNIT_ASSERT_NO_THROW(exe->setSettingsFromCLI(cli));
+
+        exe->getTreeExecutionLevel(min, max);
+        CPPUNIT_ASSERT_EQUAL(min, Index(1));
+        CPPUNIT_ASSERT_EQUAL(max, Index(2));
+        CPPUNIT_ASSERT_EQUAL(defaultCreateMissing, exe->getCreateMissing());
+        CPPUNIT_ASSERT_EQUAL(defaultTileStream, exe->getActiveTileStreaming());
+        CPPUNIT_ASSERT_EQUAL(defaultValueIter, exe->getValueIterator());
+        CPPUNIT_ASSERT_EQUAL(defaultGrain, exe->getGrainSize());
+        CPPUNIT_ASSERT_EQUAL(defaultTileGrain, exe->getActiveTileStreamingGrainSize());
+        CPPUNIT_ASSERT_EQUAL(defaultBindings, exe->getAttributeBindings());
+    }
+
+    // --tree-level
+    {
+        CPPUNIT_ASSERT_THROW(CreateCLI("--volume-grain"), openvdb::CLIError);
+        CPPUNIT_ASSERT_THROW(CreateCLI("--volume-grain invalid"), openvdb::CLIError);
+        CPPUNIT_ASSERT_THROW(CreateCLI("--volume-grain --group test"), openvdb::CLIError);
+
+        CLI cli = CreateCLI("--volume-grain 0");
+        auto exe = compiler->compile<openvdb::ax::VolumeExecutable>("");
+        CPPUNIT_ASSERT_NO_THROW(exe->setSettingsFromCLI(cli));
+
+        Index min,max;
+        exe->getTreeExecutionLevel(min, max);
+        CPPUNIT_ASSERT_EQUAL(defaultMinLevel, min);
+        CPPUNIT_ASSERT_EQUAL(defaultMaxLevel, max);
+        CPPUNIT_ASSERT_EQUAL(defaultCreateMissing, exe->getCreateMissing());
+        CPPUNIT_ASSERT_EQUAL(defaultTileStream, exe->getActiveTileStreaming());
+        CPPUNIT_ASSERT_EQUAL(defaultValueIter, exe->getValueIterator());
+        CPPUNIT_ASSERT_EQUAL(size_t(0), exe->getGrainSize());
+        CPPUNIT_ASSERT_EQUAL(defaultTileGrain, exe->getActiveTileStreamingGrainSize());
+        CPPUNIT_ASSERT_EQUAL(defaultBindings, exe->getAttributeBindings());
+
+        cli = CreateCLI("--volume-grain 1:2");
+        CPPUNIT_ASSERT_NO_THROW(exe->setSettingsFromCLI(cli));
+
+        exe->getTreeExecutionLevel(min, max);
+        CPPUNIT_ASSERT_EQUAL(defaultMinLevel, min);
+        CPPUNIT_ASSERT_EQUAL(defaultMaxLevel, max);
+        CPPUNIT_ASSERT_EQUAL(defaultCreateMissing, exe->getCreateMissing());
+        CPPUNIT_ASSERT_EQUAL(defaultTileStream, exe->getActiveTileStreaming());
+        CPPUNIT_ASSERT_EQUAL(defaultValueIter, exe->getValueIterator());
+        CPPUNIT_ASSERT_EQUAL(size_t(1), exe->getGrainSize());
+        CPPUNIT_ASSERT_EQUAL(size_t(2), exe->getActiveTileStreamingGrainSize());
+        CPPUNIT_ASSERT_EQUAL(defaultBindings, exe->getAttributeBindings());
+    }
+
+    // --bindings
+    {
+        CPPUNIT_ASSERT_THROW(CreateCLI("--bindings"), openvdb::CLIError);
+        CPPUNIT_ASSERT_THROW(CreateCLI("--bindings :"), openvdb::CLIError);
+        CPPUNIT_ASSERT_THROW(CreateCLI("--bindings ,"), openvdb::CLIError);
+        CPPUNIT_ASSERT_THROW(CreateCLI("--bindings a:"), openvdb::CLIError);
+        CPPUNIT_ASSERT_THROW(CreateCLI("--bindings a,b"), openvdb::CLIError);
+        CPPUNIT_ASSERT_THROW(CreateCLI("--bindings :b"), openvdb::CLIError);
+        CPPUNIT_ASSERT_THROW(CreateCLI("--bindings ,a:b"), openvdb::CLIError);
+        CPPUNIT_ASSERT_THROW(CreateCLI("--bindings --create-missing ON"), openvdb::CLIError);
+
+        CLI cli = CreateCLI("--bindings a:b,c:d,12:13");
+        ax::AttributeBindings bindings;
+        bindings.set("a", "b");
+        bindings.set("c", "d");
+        bindings.set("12", "13");
+
+        auto exe = compiler->compile<openvdb::ax::VolumeExecutable>("");
+        CPPUNIT_ASSERT_NO_THROW(exe->setSettingsFromCLI(cli));
+
+        Index min,max;
+        exe->getTreeExecutionLevel(min, max);
+        CPPUNIT_ASSERT_EQUAL(defaultMinLevel, min);
+        CPPUNIT_ASSERT_EQUAL(defaultMaxLevel, max);
+        CPPUNIT_ASSERT_EQUAL(defaultCreateMissing, exe->getCreateMissing());
+        CPPUNIT_ASSERT_EQUAL(defaultTileStream, exe->getActiveTileStreaming());
+        CPPUNIT_ASSERT_EQUAL(defaultValueIter, exe->getValueIterator());
+        CPPUNIT_ASSERT_EQUAL(defaultGrain, exe->getGrainSize());
+        CPPUNIT_ASSERT_EQUAL(defaultTileGrain, exe->getActiveTileStreamingGrainSize());
+        CPPUNIT_ASSERT_EQUAL(bindings, exe->getAttributeBindings());
+    }
+
+    // multiple
+    {
+        CLI cli = CreateCLI("--volume-grain 5:10 --create-missing OFF");
+        auto exe = compiler->compile<openvdb::ax::VolumeExecutable>("");
+        CPPUNIT_ASSERT_NO_THROW(exe->setSettingsFromCLI(cli));
+
+        Index min,max;
+        exe->getTreeExecutionLevel(min, max);
+        CPPUNIT_ASSERT_EQUAL(defaultMinLevel, min);
+        CPPUNIT_ASSERT_EQUAL(defaultMaxLevel, max);
+        CPPUNIT_ASSERT_EQUAL(false, exe->getCreateMissing());
+        CPPUNIT_ASSERT_EQUAL(defaultTileStream, exe->getActiveTileStreaming());
+        CPPUNIT_ASSERT_EQUAL(defaultValueIter, exe->getValueIterator());
+        CPPUNIT_ASSERT_EQUAL(size_t(5), exe->getGrainSize());
+        CPPUNIT_ASSERT_EQUAL(size_t(10), exe->getActiveTileStreamingGrainSize());
+        CPPUNIT_ASSERT_EQUAL(defaultBindings, exe->getAttributeBindings());
+
+        cli = CreateCLI("--tile-stream ON --node-iter OFF --tree-level 2:3 --volume-grain 10:20 --create-missing ON --bindings a:b");
+        ax::AttributeBindings bindings;
+        bindings.set("a", "b");
+
+        CPPUNIT_ASSERT_NO_THROW(exe->setSettingsFromCLI(cli));
+        exe->getTreeExecutionLevel(min, max);
+        CPPUNIT_ASSERT_EQUAL(Index(2), min);
+        CPPUNIT_ASSERT_EQUAL(Index(3), max);
+        CPPUNIT_ASSERT_EQUAL(true, exe->getCreateMissing());
+        CPPUNIT_ASSERT_EQUAL(openvdb::ax::VolumeExecutable::Streaming::ON, exe->getActiveTileStreaming());
+        CPPUNIT_ASSERT_EQUAL(openvdb::ax::VolumeExecutable::IterType::OFF, exe->getValueIterator());
+        CPPUNIT_ASSERT_EQUAL(size_t(10), exe->getGrainSize());
+        CPPUNIT_ASSERT_EQUAL(size_t(20), exe->getActiveTileStreamingGrainSize());
+        CPPUNIT_ASSERT_EQUAL(bindings, exe->getAttributeBindings());
+    }
+}
