@@ -1,5 +1,5 @@
 // Copyright Contributors to the OpenVDB Project
-// SPDX-License-Identifier: MPL-2.0
+// SPDX-License-Identifier: Apache-2.0
 
 #include "TestHarness.h"
 #include "../util.h"
@@ -368,6 +368,12 @@ void
 TestVDBFunctions::testValidContext()
 {
     std::shared_ptr<llvm::LLVMContext> C(new llvm::LLVMContext);
+#if LLVM_VERSION_MAJOR >= 15
+    // This will not work from LLVM 16. We'll need to fix this
+    // https://llvm.org/docs/OpaquePointers.html
+    C->setOpaquePointers(false);
+#endif
+
     openvdb::ax::Compiler compiler;
     openvdb::ax::FunctionOptions ops;
     ops.mLazyFunctions = false;
