@@ -1,5 +1,5 @@
 // Copyright Contributors to the OpenVDB Project
-// SPDX-License-Identifier: MPL-2.0
+// SPDX-License-Identifier: Apache-2.0
 
 /// @file SOP_OpenVDB_Points_Convert.cc
 ///
@@ -615,7 +615,7 @@ SOP_OpenVDB_Points_Convert::Cache::cookVDBSop(OP_Context& context)
                     matchGroup(*refGeo, evalStdString("refvdb", time));
 
                 hvdb::VdbPrimCIterator it(refGeo, refGroup);
-                const hvdb::GU_PrimVDB* refPrim = *it;
+                const GU_PrimVDB* refPrim = *it;
 
                 if (!refPrim) {
                     addError(SOP_MESSAGE, "Second input has no VDB primitives.");
@@ -806,7 +806,7 @@ SOP_OpenVDB_Points_Convert::Cache::cookVDBSop(OP_Context& context)
 
         // Unpack any packed primitives
 
-        std::unique_ptr<GA_PrimitiveGroup> packgroup(ptGeo->newDetachedPrimitiveGroup());
+        GA_PrimitiveGroupUPtr packgroup = ptGeo->createDetachedPrimitiveGroup();
 
         for (GA_Iterator it(ptGeo->getPrimitiveRange()); !it.atEnd(); ++it) {
             GA_Offset offset = *it;
